@@ -53,69 +53,40 @@ typedef struct {
 
 
 CommentData *  comment_data_new                    (void);
-
 void           comment_data_free                   (CommentData *data);
-
 CommentData *  comment_data_dup                    (CommentData *data);
-
 void           comment_data_free_keywords          (CommentData *data);
-
 void           comment_data_free_comment           (CommentData *data);
-
 void           comment_data_remove_keyword         (CommentData *data,
 						    const char  *keyword);
-
 void           comment_data_add_keyword            (CommentData *data,
 						    const char  *keyword);
-
 gboolean       comment_data_is_void                (CommentData *data);
-
 gboolean       comment_text_is_void                (CommentData *data);
 
 /* -- */
 
 char *         comments_get_comment_filename       (const char  *source,
 						    gboolean     resolve_symlinks);
-
-char *         comments_get_comment_dir            (const char  *directory,
-						    gboolean     resolve_symlinks);
-
 void           comment_copy                        (const char  *src,
 						    const char  *dest);
-
 void           comment_move                        (const char  *src,
 						    const char  *dest);
-
-void           comment_delete                      (const char  *filename);
-
-void           comments_remove_old_comments        (const char  *dir,
-						    gboolean     recursive,
-						    gboolean     clear_all);
-
-void           comments_remove_old_comments_async  (const char  *dir,
-						    gboolean     recursive,
-						    gboolean     clear_all);
-
-CommentData *  comments_load_comment               (const char  *filename,
+void           comment_delete                      (const char  *uri);
+CommentData *  comments_load_comment               (const char  *uri,
 						    gboolean     try_embedded);
-
-void           comments_save_comment               (const char  *filename,
+void           comments_save_comment               (const char  *uri,
 						    CommentData *data);
-
-void           comments_save_comment_non_null      (const char  *filename,
+void           comments_save_comment_non_null      (const char  *uri,
 						    CommentData *data);
-
-void           comments_save_categories            (const char  *filename,
+void           comments_save_categories            (const char  *uri,
 						    CommentData *data);
-
 char *         comments_get_comment_as_string      (CommentData *data,
 						    char        *sep1,
 						    char        *sep2);
-
 char *         comments_get_comment_as_xml_string  (CommentData *data,
 						    char        *sep1,
 						    char        *sep2);
-
 char*          _g_escape_text_for_html             (const gchar *text,
 						    gssize       length);
 
