@@ -26,7 +26,6 @@
 #include <glib/gi18n.h>
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/gnome-icon-lookup.h>
 
 #include "typedefs.h"
 #include "gth-file-view.h"
@@ -35,9 +34,8 @@
 #include "gthumb-enum-types.h"
 #include "file-utils.h"
 #include "file-data.h"
-#include "icons/pixbufs.h"
-#include "pixbuf-utils.h"
 #include "gth-sort-utils.h"
+#include "pixbuf-utils.h"
 
 
 enum {
@@ -48,6 +46,7 @@ enum {
 	COLUMN_SIZE,
 	COLUMN_TIME,
 	COLUMN_COMMENT,
+	COLUMN_EXIF_DATE,
 	NUMBER_OF_COLUMNS
 };
 
@@ -62,11 +61,9 @@ struct _GthFileViewListPrivate {
 	GthVisibleFunc  filter_func;
 	gpointer        filter_data;
 	int             max_image_size;
-	GtkIconTheme   *icon_theme;
 	gboolean        enable_thumbs;
 	gboolean        reorderable;
-	GdkPixbuf      *unknown_pixbuf_small;
-	GdkPixbuf      *unknown_pixbuf_big;
+	GHashTable     *scaled_pixbufs;
 };
 
 
@@ -139,14 +136,27 @@ get_sized_pixbuf (GthFileViewList *gfv_list,
 		  GdkPixbuf       *pixbuf)
 {
 	GdkPixbuf *result;
+	int        max_image_size;
 	int        x, y, w, h;
 
 	if (gfv_list->priv->max_image_size == 0)
 		return NULL;
 
+	/**/
+
+	max_image_size = gfv_list->priv->max_image_size;
+
+	if (! gfv_list->priv->enable_thumbs) {
+		int icon_width, icon_height;
+		gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (GTK_WIDGET (gfv_list->priv->tree_view)),
+						   GTK_ICON_SIZE_MENU,
+						   &icon_width, &icon_height);
+		max_image_size = MAX (icon_width, icon_height);
+	}
+
 	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				 gfv_list->priv->max_image_size,
-				 gfv_list->priv->max_image_size);
+				 max_image_size,
+				 max_image_size);
 	gdk_pixbuf_fill (result, 0x00000000);
 
 	if (pixbuf == NULL)
@@ -154,14 +164,46 @@ get_sized_pixbuf (GthFileViewList *gfv_list,
 
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
-	x = (gfv_list->priv->max_image_size - w) / 2;
-	y = (gfv_list->priv->max_image_size - h) / 2;
+	if (scale_keepping_ratio (&w, &h, max_image_size, max_image_size)) {
+		GdkPixbuf *try_pixbuf;
 
-	gdk_pixbuf_copy_area (pixbuf,
-			      0, 0,
-			      w, h,
-			      result,
-			      x, y);
+		/* look in the hash table. */
+
+		try_pixbuf = g_hash_table_lookup (gfv_list->priv->scaled_pixbufs, pixbuf);
+		if (try_pixbuf != NULL) {
+			g_object_unref (result);
+			result = try_pixbuf;
+			g_object_ref (G_OBJECT (result));
+			return result;
+		}
+		else {
+			GdkPixbuf *scaled;
+
+			scaled = gdk_pixbuf_scale_simple (pixbuf,
+							  w,
+							  h,
+							  GDK_INTERP_BILINEAR);
+			x = (max_image_size - w) / 2;
+			y = (max_image_size - h) / 2;
+			gdk_pixbuf_copy_area (scaled,
+					      0, 0,
+					      w, h,
+					      result,
+					      x, y);
+
+			g_hash_table_insert (gfv_list->priv->scaled_pixbufs, pixbuf, scaled);
+		}
+	}
+	else {
+		x = (max_image_size - w) / 2;
+		y = (max_image_size - h) / 2;
+
+		gdk_pixbuf_copy_area (pixbuf,
+				      0, 0,
+				      w, h,
+				      result,
+				      x, y);
+	}
 
 	return result;
 }
@@ -179,12 +221,9 @@ gfv_insert (GthFileView  *file_view,
 	GtkTreeIter      iter;
 	GdkPixbuf       *real_pixbuf;
 
-	if (! gfv_list->priv->enable_thumbs)
-		real_pixbuf = g_object_ref (gfv_list->priv->unknown_pixbuf_small);
-	else if (pixbuf == NULL)
-		real_pixbuf = get_sized_pixbuf (gfv_list, gfv_list->priv->unknown_pixbuf_big);
-	else
-		real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
+	g_return_if_fail (pixbuf != NULL);
+
+	real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
 
 	gtk_list_store_insert (list_store, &iter, pos);
 	gtk_list_store_set (list_store, &iter,
@@ -211,12 +250,9 @@ gfv_append (GthFileView  *file_view,
 	GtkTreePath     *path;
 	int              pos;
 
-	if (! gfv_list->priv->enable_thumbs)
-		real_pixbuf = g_object_ref (gfv_list->priv->unknown_pixbuf_small);
-	else if (pixbuf == NULL)
-		real_pixbuf = get_sized_pixbuf (gfv_list, gfv_list->priv->unknown_pixbuf_big);
-	else
-		real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
+	g_return_val_if_fail (pixbuf != NULL, -1);
+
+	real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
 
 	gtk_list_store_append (list_store, &iter);
 	gtk_list_store_set (list_store, &iter,
@@ -250,12 +286,9 @@ gfv_append_with_data (GthFileView  *file_view,
 	GtkTreePath     *path;
 	int              pos;
 
-	if (! gfv_list->priv->enable_thumbs)
-		real_pixbuf = g_object_ref (gfv_list->priv->unknown_pixbuf_small);
-	else if (pixbuf == NULL)
-		real_pixbuf = get_sized_pixbuf (gfv_list, gfv_list->priv->unknown_pixbuf_big);
-	else
-		real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
+	g_return_val_if_fail (pixbuf != NULL, -1);
+
+	real_pixbuf = get_sized_pixbuf (gfv_list, pixbuf);
 
 	gtk_list_store_append (list_store, &iter);
 	gtk_list_store_set (list_store, &iter,
@@ -280,12 +313,12 @@ static void
 gfv_remove (GthFileView *file_view,
 	    gpointer     data)
 {
-	/* FIXME */
-	/*
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
 	GtkTreePath     *path;
+	int              pos;
 
+	pos = gth_file_view_find_image_from_data (file_view, data);
 	path = gtk_tree_path_new_from_indices (pos, -1);
 	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
 				       &iter,
@@ -296,7 +329,6 @@ gfv_remove (GthFileView *file_view,
 	gtk_tree_path_free (path);
 
 	gtk_list_store_remove (gfv_list->priv->list_store, &iter);
-	*/
 }
 
 
@@ -308,7 +340,7 @@ gfv_clear (GthFileView  *file_view)
 	gtk_list_store_clear (gfv_list->priv->list_store);
 
 	if (GTK_WIDGET_REALIZED (gfv_list->priv->tree_view))
-                gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (gfv_list->priv->tree_view), 0, 0);
+		gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (gfv_list->priv->tree_view), 0, 0);
 }
 
 
@@ -352,31 +384,7 @@ gfv_set_image_pixbuf (GthFileView  *file_view,
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_ICON, get_sized_pixbuf (gfv_list, pixbuf),
-                            -1);
-}
-
-
-static void
-gfv_set_unknown_pixbuf (GthFileView  *file_view,
-			int           pos)
-{
-	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
-	GtkTreeIter      iter;
-	GdkPixbuf       *real_pixbuf;
-
-	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
-		return;
-
-	if (! gfv_list->priv->enable_thumbs)
-		real_pixbuf = g_object_ref (gfv_list->priv->unknown_pixbuf_small);
-	else
-		real_pixbuf = get_sized_pixbuf (gfv_list, gfv_list->priv->unknown_pixbuf_big);
-
-	gtk_list_store_set (gfv_list->priv->list_store, &iter,
-			    COLUMN_ICON, real_pixbuf,
-                            -1);
-
-	g_object_unref (real_pixbuf);
+			    -1);
 }
 
 
@@ -393,7 +401,7 @@ gfv_set_image_text (GthFileView  *file_view,
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_NAME, text,
-                            -1);
+			    -1);
 }
 
 
@@ -416,8 +424,8 @@ gfv_get_image_text (GthFileView  *file_view,
 	gtk_tree_path_free (path);
 
 	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->filter_model), &iter,
-                            COLUMN_NAME, &text,
-                            -1);
+			    COLUMN_NAME, &text,
+			    -1);
 
 	return text;
 }
@@ -436,7 +444,7 @@ gfv_set_image_comment (GthFileView  *file_view,
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_COMMENT, comment,
-                            -1);
+			    -1);
 }
 
 
@@ -650,46 +658,6 @@ gfv_pos_is_selected (GthFileView     *file_view,
 }
 
 
-static gboolean
-gfv_only_one_is_selected (GthFileView    *file_view)
-{
-	GthFileViewList  *gfv_list = (GthFileViewList *) file_view;
-	GtkTreeSelection *selection;
-	GList            *sel_rows;
-	gboolean          ret_val;
-
-	selection = gtk_tree_view_get_selection (gfv_list->priv->tree_view);
-	sel_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
-
-	ret_val = ((sel_rows != NULL) && (sel_rows->next == NULL));
-
-	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (sel_rows);
-
-	return ret_val;
-}
-
-
-static gboolean
-gfv_selection_not_null (GthFileView    *file_view)
-{
-	GthFileViewList  *gfv_list = (GthFileViewList *) file_view;
-	GtkTreeSelection *selection;
-	GList            *sel_rows;
-	gboolean          ret_val;
-
-	selection = gtk_tree_view_get_selection (gfv_list->priv->tree_view);
-	sel_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
-
-	ret_val = (sel_rows != NULL);
-
-	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (sel_rows);
-
-	return ret_val;
-}
-
-
 static int
 gfv_get_first_selected (GthFileView *file_view)
 {
@@ -754,6 +722,26 @@ gfv_get_last_selected (GthFileView *file_view)
 }
 
 
+static int
+gfv_get_n_selected (GthFileView *file_view)
+{
+	GthFileViewList  *gfv_list = (GthFileViewList *) file_view;
+	GtkTreeSelection *selection;
+	GList            *sel_rows;
+	int               n_selected;
+
+	selection = gtk_tree_view_get_selection (gfv_list->priv->tree_view);
+	sel_rows = gtk_tree_selection_get_selected_rows (selection, NULL);
+
+	n_selected = g_list_length (sel_rows);
+
+	g_list_foreach (sel_rows, (GFunc) gtk_tree_path_free, NULL);
+	g_list_free (sel_rows);
+
+	return n_selected;	
+}
+
+
 /* Setting spacing values */
 
 
@@ -782,18 +770,7 @@ gfv_set_image_data (GthFileView     *file_view,
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_FILE_DATA, data,
-                            -1);
-}
-
-
-static void
-gfv_set_image_data_full (GthFileView     *file_view,
-			 int              pos,
-			 gpointer         data,
-			 GtkDestroyNotify destroy)
-{
-	/* FIXME: not implemented yet */
-	gth_file_view_set_image_data (file_view, pos, data);
+			    -1);
 }
 
 
@@ -815,9 +792,12 @@ gfv_find_image_from_data (GthFileView     *file_view,
 		gtk_tree_model_get (model, &iter,
 				    COLUMN_FILE_DATA, &iter_data,
 				    -1);
-		if (data == iter_data)
+		if (data == iter_data) {
+			file_data_unref (iter_data);
 			return pos;
-
+		}
+		file_data_unref (iter_data);
+		
 		pos++;
 	} while (gtk_tree_model_iter_next (model, &iter));
 
@@ -844,9 +824,8 @@ gfv_get_image_data (GthFileView     *file_view,
 	gtk_tree_path_free (path);
 
 	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->filter_model), &iter,
-                            COLUMN_FILE_DATA, &fdata,
-                            -1);
-	file_data_ref (fdata);
+			    COLUMN_FILE_DATA, &fdata,
+			    -1);
 
 	return fdata;
 }
@@ -873,13 +852,8 @@ gfv_set_view_mode (GthFileView *file_view,
 
 	gfv_list->priv->view_mode = mode;
 
-	column = gtk_tree_view_get_column (gfv_list->priv->tree_view, 1);
-
-	/* FIXME
-        gtk_tree_view_column_set_visible (column, mode == GTH_VIEW_MODE_ALL);
-	*/
-
-        gtk_tree_view_column_set_visible (column, FALSE);
+	column = gtk_tree_view_get_column (gfv_list->priv->tree_view, 2);
+	gtk_tree_view_column_set_visible (column, FALSE /*mode == GTH_VIEW_MODE_ALL*/);
 }
 
 
@@ -1023,27 +997,31 @@ gfv_get_last_visible (GthFileView *file_view)
 
 static gboolean
 gfv_visible_func (GtkTreeModel *model,
-                  GtkTreeIter  *iter,
-                  gpointer      data)
+		  GtkTreeIter  *iter,
+		  gpointer      data)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) data;
 	FileData        *fdata;
-
+	gboolean         visible;
+	
 	if (gfv_list->priv->filter_func == NULL)
 		return TRUE;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->list_store), iter,
-                            COLUMN_FILE_DATA, &fdata,
-                            -1);
+			    COLUMN_FILE_DATA, &fdata,
+			    -1);
 
-	return (gfv_list->priv->filter_func) (gfv_list->priv->filter_data, fdata);
+	visible = (gfv_list->priv->filter_func) (gfv_list->priv->filter_data, fdata);
+	file_data_unref (fdata);
+	
+	return visible;
 }
 
 
 static void
 gfv_set_visible_func (GthFileView    *file_view,
-                      GthVisibleFunc  func,
-                      gpointer        data)
+		      GthVisibleFunc  func,
+		      gpointer        data)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 
@@ -1072,6 +1050,7 @@ gfv_sorted (GthFileView   *file_view,
 	case GTH_SORT_METHOD_BY_SIZE: col = COLUMN_SIZE; break;
 	case GTH_SORT_METHOD_BY_TIME: col = COLUMN_TIME; break;
 	case GTH_SORT_METHOD_BY_COMMENT: col = COLUMN_COMMENT; break;
+	case GTH_SORT_METHOD_BY_EXIF_DATE: col = COLUMN_EXIF_DATE; break;
 	default: col = GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID; break;
 	}
 
@@ -1146,80 +1125,6 @@ gfv_get_cursor (GthFileView *file_view)
 }
 
 
-static GdkPixbuf *
-create_unknown_pixbuf (GthFileViewList *gfv_list, gboolean big)
-{
-	GtkIconTheme   *icon_theme = gfv_list->priv->icon_theme;
-	GtkIconInfo    *icon_info = NULL;
-	int             icon_width, icon_height, icon_size;
-	char           *icon_name;
-	GdkPixbuf      *pixbuf = NULL;
-	int             width, height;
-
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (GTK_WIDGET (gfv_list->priv->tree_view)),
-                                           (big ? GTK_ICON_SIZE_DIALOG: GTK_ICON_SIZE_LARGE_TOOLBAR),
-                                           &icon_width, &icon_height);
-	icon_size = MAX (icon_width, icon_height);
-
-	icon_name = gnome_icon_lookup (icon_theme,
-				       NULL,
-				       NULL,
-				       NULL,
-				       NULL,
-				       "image/*",
-				       GNOME_ICON_LOOKUP_FLAGS_NONE,
-				       NULL);
-
-	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
-						icon_name,
-						icon_size,
-						0);
-
-	g_free (icon_name);
-
-	if (icon_info != NULL) {
-		pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
-		gtk_icon_info_free (icon_info);
-	}
-
-	if (pixbuf == NULL)
-		pixbuf = gdk_pixbuf_new_from_inline (-1,
-						     dir_16_rgba,
-						     FALSE,
-						     NULL);
-
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	if (scale_keepping_ratio (&width, &height, icon_size, icon_size)) {
-		GdkPixbuf *scaled;
-		scaled = gdk_pixbuf_scale_simple (pixbuf,
-						  width,
-						  height,
-						  GDK_INTERP_BILINEAR);
-		g_object_unref (pixbuf);
-		pixbuf = scaled;
-	}
-
-	return pixbuf;
-}
-
-
-static void
-gfv_update_icon_theme (GthFileView *file_view)
-{
-	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
-
-	if (gfv_list->priv->unknown_pixbuf_small != NULL)
-		g_object_unref (gfv_list->priv->unknown_pixbuf_small);
-
-	if (gfv_list->priv->unknown_pixbuf_big != NULL)
-		g_object_unref (gfv_list->priv->unknown_pixbuf_big);
-
-	gfv_list->priv->unknown_pixbuf_small = create_unknown_pixbuf (gfv_list, FALSE);
-	gfv_list->priv->unknown_pixbuf_big = create_unknown_pixbuf (gfv_list, TRUE);
-}
-
-
 /* DnD */
 
 
@@ -1331,6 +1236,29 @@ gfv_get_enable_search (GthFileView *file_view)
 /**/
 
 
+static void
+gh_unref_pixbuf (gpointer  key,
+		 gpointer  value,
+		 gpointer  user_data)
+{
+	g_object_unref (value);
+}
+
+
+static void
+gth_file_list_free_pixbufs (GthFileViewList *gfv_list)
+{
+	if (gfv_list->priv->scaled_pixbufs == NULL)
+		return;
+
+	g_hash_table_foreach (gfv_list->priv->scaled_pixbufs,
+			      gh_unref_pixbuf,
+			      NULL);
+	g_hash_table_destroy (gfv_list->priv->scaled_pixbufs);
+	gfv_list->priv->scaled_pixbufs = NULL;
+}
+
+
 static void
 gth_file_view_list_finalize (GObject *object)
 {
@@ -1339,12 +1267,10 @@ gth_file_view_list_finalize (GObject *object)
 
 	gfv_list = (GthFileViewList*) object;
 
-	g_object_unref (gfv_list->priv->unknown_pixbuf_small);
-	g_object_unref (gfv_list->priv->unknown_pixbuf_big);
-
+	gth_file_list_free_pixbufs (gfv_list);
 	g_free (gfv_list->priv);
 
-        /* Chain up */
+	/* Chain up */
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1375,7 +1301,6 @@ gth_file_view_list_class_init (GthFileViewListClass *file_view_list_class)
 	file_view_class->remove               = gfv_remove;
 	file_view_class->clear                = gfv_clear;
 	file_view_class->set_image_pixbuf     = gfv_set_image_pixbuf;
-	file_view_class->set_unknown_pixbuf   = gfv_set_unknown_pixbuf;
 	file_view_class->set_image_text       = gfv_set_image_text;
 	file_view_class->get_image_text       = gfv_get_image_text;
 	file_view_class->set_image_comment    = gfv_set_image_comment;
@@ -1389,13 +1314,11 @@ gth_file_view_list_class_init (GthFileViewListClass *file_view_list_class)
 	file_view_class->unselect_all         = gfv_unselect_all;
 	file_view_class->get_file_list_selection = gfv_get_file_list_selection;
 	file_view_class->pos_is_selected      = gfv_pos_is_selected;
-	file_view_class->only_one_is_selected = gfv_only_one_is_selected;
-	file_view_class->selection_not_null   = gfv_selection_not_null;
 	file_view_class->get_first_selected   = gfv_get_first_selected;
 	file_view_class->get_last_selected    = gfv_get_last_selected;
+	file_view_class->get_n_selected       = gfv_get_n_selected;	
 	file_view_class->set_image_width      = gfv_set_image_width;
 	file_view_class->set_image_data       = gfv_set_image_data;
-	file_view_class->set_image_data_full  = gfv_set_image_data_full;
 	file_view_class->find_image_from_data = gfv_find_image_from_data;
 	file_view_class->get_image_data       = gfv_get_image_data;
 	file_view_class->enable_thumbs        = gfv_enable_thumbs;
@@ -1412,7 +1335,6 @@ gth_file_view_list_class_init (GthFileViewListClass *file_view_list_class)
 	file_view_class->image_activated      = gfv_image_activated;
 	file_view_class->set_cursor           = gfv_set_cursor;
 	file_view_class->get_cursor           = gfv_get_cursor;
-	file_view_class->update_icon_theme    = gfv_update_icon_theme;
 	file_view_class->set_drag_dest_pos    = gfv_set_drag_dest_pos;
 	file_view_class->get_drag_dest_pos    = gfv_get_drag_dest_pos;
 	file_view_class->set_reorderable      = gfv_set_reorderable;
@@ -1433,6 +1355,8 @@ gth_file_view_list_init (GthFileViewList *gfv_list)
 	priv->sort_method = GTH_SORT_METHOD_NONE;
 	priv->sort_type   = GTK_SORT_ASCENDING;
 	priv->reorderable = FALSE;
+
+	priv->scaled_pixbufs = g_hash_table_new (g_direct_hash, NULL);
 }
 
 
@@ -1452,13 +1376,13 @@ gth_file_view_list_get_type (void)
 			sizeof (GthFileViewList),
 			0,
 			(GInstanceInitFunc) gth_file_view_list_init
-                };
+		};
 
 		type = g_type_register_static (GTH_TYPE_FILE_VIEW,
 					       "GthFileViewList",
 					       &type_info,
 					       0);
-        }
+	}
 
 	return type;
 }
@@ -1502,57 +1426,69 @@ cursor_changed_cb (GtkTreeView     *tree_view,
 static void
 add_columns (GtkTreeView *treeview)
 {
-	static char       *titles[] = {N_("Comment")};
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	int                i, j;
 	GValue             value = { 0, };
 
-	/* The Name column. */
+	/* The icon column */
+
 	column = gtk_tree_view_column_new ();
-	gtk_tree_view_column_set_title (column, _("Name"));
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-                                             "pixbuf", COLUMN_ICON,
-                                             NULL);
+					     "pixbuf", COLUMN_ICON,
+					     NULL);
+
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_column_set_expand (column, FALSE);
+
+	gtk_tree_view_append_column (treeview, column);
+
+	/* The name column */
+
+	column = gtk_tree_view_column_new ();
 
 	renderer = gtk_cell_renderer_text_new ();
-        g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
-        g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
-        g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
-        g_value_unset (&value);
-
-	gtk_tree_view_column_pack_start (column,
-					 renderer,
-					 TRUE);
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-                                             "text", COLUMN_NAME,
-                                             NULL);
+					     "text", COLUMN_NAME,
+					     NULL);
+
+	g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+	g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+	g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+	g_value_unset (&value);
 
-	/*gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);*/
-	/*
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
-	gtk_tree_view_column_set_sort_column_id (column, COLUMN_PATH);
-	gtk_tree_view_column_set_sort_column_id (column, COLUMN_SIZE);
-	gtk_tree_view_column_set_sort_column_id (column, COLUMN_TIME);
-	*/
-	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_column_set_expand (column, TRUE);
 
-	/* Other columns */
-	for (j = 0, i = COLUMN_COMMENT; i < NUMBER_OF_COLUMNS; i++, j++) {
-		renderer = gtk_cell_renderer_text_new ();
-		column = gtk_tree_view_column_new_with_attributes (_(titles[j]),
-								   renderer,
-								   "text", i,
-								   NULL);
+	gtk_tree_view_append_column (treeview, column);
 
-		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-		gtk_tree_view_column_set_sort_column_id (column, i);
+	/* Comment column */
 
-		gtk_tree_view_append_column (treeview, column);
-	}
+	column = gtk_tree_view_column_new ();
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_set_attributes (column, renderer,
+					     "text", COLUMN_COMMENT,
+					     NULL);
+
+	g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+	g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+	g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+	g_value_unset (&value);
+
+	gtk_tree_view_column_set_sort_column_id (column, COLUMN_COMMENT);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_column_set_expand (column, TRUE);
+
+	gtk_tree_view_append_column (treeview, column);
 }
 
 
@@ -1679,8 +1615,8 @@ default_sort_func (GtkTreeModel *model,
 	GCompareFunc     cmp_func;
 	int              ret_val;
 
-        gtk_tree_model_get (model, a, COLUMN_FILE_DATA, &fdata1, -1);
-        gtk_tree_model_get (model, b, COLUMN_FILE_DATA, &fdata2, -1);
+	gtk_tree_model_get (model, a, COLUMN_FILE_DATA, &fdata1, -1);
+	gtk_tree_model_get (model, b, COLUMN_FILE_DATA, &fdata2, -1);
 
 	g_return_val_if_fail (fdata1 != NULL, 0);
 	g_return_val_if_fail (fdata2 != NULL, 0);
@@ -1688,6 +1624,9 @@ default_sort_func (GtkTreeModel *model,
 	cmp_func = get_compfunc_from_method (gfv_list->priv->sort_method);
 	ret_val = (*cmp_func) (fdata1, fdata2);
 
+	file_data_unref (fdata1);
+	file_data_unref (fdata2);
+
 	return ret_val;
 }
 
@@ -1703,12 +1642,13 @@ gth_file_view_list_new (guint image_width)
 	priv = gfv_list->priv;
 
 	priv->list_store = gtk_list_store_new (NUMBER_OF_COLUMNS,
-					       G_TYPE_POINTER,
+					       GTH_TYPE_FILE_DATA,
 					       GDK_TYPE_PIXBUF,
 					       G_TYPE_STRING,
 					       G_TYPE_STRING,
 					       G_TYPE_STRING,
 					       G_TYPE_STRING,
+					       G_TYPE_STRING,
 					       G_TYPE_STRING);
 	priv->filter_model =  gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->list_store), NULL);
 	g_object_unref (priv->list_store);
@@ -1729,16 +1669,23 @@ gth_file_view_list_new (guint image_width)
 						 gfv_list, NULL);
 
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->list_store),
-                                         COLUMN_NAME, default_sort_func,
-                                         gfv_list, NULL);
+					 COLUMN_NAME, default_sort_func,
+					 gfv_list, NULL);
+
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->list_store),
-                                         COLUMN_PATH, default_sort_func,
-                                         gfv_list, NULL);
+					 COLUMN_PATH, default_sort_func,
+					 gfv_list, NULL);
+
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->list_store),
-                                         COLUMN_SIZE, default_sort_func,
-                                         gfv_list, NULL);
+					 COLUMN_SIZE, default_sort_func,
+					 gfv_list, NULL);
+
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->list_store),
-                                         COLUMN_TIME, default_sort_func,
+					 COLUMN_TIME, default_sort_func,
+					 gfv_list, NULL);
+
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->list_store),
+                                         COLUMN_EXIF_DATE, default_sort_func,
                                          gfv_list, NULL);
 
 	selection = gtk_tree_view_get_selection (priv->tree_view);
@@ -1762,11 +1709,5 @@ gth_file_view_list_new (guint image_width)
 			  G_CALLBACK (cursor_changed_cb),
 			  gfv_list);
 
-	/**/
-
-	priv->icon_theme = gtk_icon_theme_get_default ();
-	priv->unknown_pixbuf_small = create_unknown_pixbuf (gfv_list, FALSE);
-	priv->unknown_pixbuf_big = create_unknown_pixbuf (gfv_list, TRUE);
-
 	return GTH_FILE_VIEW (gfv_list);
 }
