@@ -22,21 +22,18 @@
 
 #include <string.h>
 #include <glib.h>
-#include <gnome.h>
 #include "gth-file-view.h"
 #include "gth-file-view-thumbs.h"
 #include "gthumb-marshal.h"
 #include "gthumb-enum-types.h"
 #include "gth-image-list.h"
+#include "file-utils.h"
 #include "file-data.h"
-#include "icons/pixbufs.h"
-#include "pixbuf-utils.h"
+#include "gth-sort-utils.h"
 
 
 struct _GthFileViewThumbsPrivate {
 	GthImageList   *ilist;
-	GnomeIconTheme *icon_theme;
-	GdkPixbuf      *unknown_pixbuf;
 };
 
 
@@ -113,7 +110,7 @@ gfv_thaw (GthFileView  *file_view)
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
 	GthImageList      *ilist = gfv_thumbs->priv->ilist;
 
-	gth_image_list_thaw (ilist);
+	gth_image_list_thaw (ilist, TRUE);
 }
 
 
@@ -132,21 +129,17 @@ gfv_is_frozen (GthFileView  *file_view)
 
 static void
 gfv_insert (GthFileView  *file_view,
-	    int           pos, 
+	    int           pos,
 	    GdkPixbuf    *pixbuf,
 	    const char   *text,
 	    const char   *comment)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
 	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-	GdkPixbuf         *real_pixbuf;
 
-	if (pixbuf == NULL)
-		real_pixbuf = gfv_thumbs->priv->unknown_pixbuf;
-	else
-		real_pixbuf = pixbuf;
+	g_return_if_fail (pixbuf != NULL);
 
-	gth_image_list_insert (ilist, pos, real_pixbuf, text, comment);
+	gth_image_list_insert (ilist, pos, pixbuf, text, comment);
 }
 
 
@@ -158,14 +151,10 @@ gfv_append (GthFileView  *file_view,
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
 	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-	GdkPixbuf         *real_pixbuf;
 
-	if (pixbuf == NULL)
-		real_pixbuf = gfv_thumbs->priv->unknown_pixbuf;
-	else
-		real_pixbuf = pixbuf;
+	g_return_val_if_fail (pixbuf != NULL, -1);
 
-	return 	gth_image_list_append (ilist, real_pixbuf, text, comment);
+	return 	gth_image_list_append (ilist, pixbuf, text, comment);
 }
 
 
@@ -178,25 +167,21 @@ gfv_append_with_data (GthFileView  *file_view,
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
 	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-	GdkPixbuf         *real_pixbuf;
 
-	if (pixbuf == NULL)
-		real_pixbuf = gfv_thumbs->priv->unknown_pixbuf;
-	else
-		real_pixbuf = pixbuf;
+	g_return_val_if_fail (pixbuf != NULL, -1);
 
-	return 	gth_image_list_append_with_data (ilist, real_pixbuf, text, comment, data);
+	return 	gth_image_list_append_with_data (ilist, pixbuf, text, comment, data);
 }
 
 
 static void
-gfv_remove (GthFileView  *file_view, 
-	    int           pos)
+gfv_remove (GthFileView  *file_view,
+	    gpointer      data)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
 	GthImageList      *ilist = gfv_thumbs->priv->ilist;
 
-	gth_image_list_remove (ilist, pos);
+	gth_image_list_remove (ilist, data);
 }
 
 
@@ -222,17 +207,6 @@ gfv_set_image_pixbuf (GthFileView  *file_view,
 }
 
 
-static void
-gfv_set_unknown_pixbuf (GthFileView  *file_view,
-			int           pos)
-{
-	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
-	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-
-	gth_image_list_set_image_pixbuf (ilist, pos, gfv_thumbs->priv->unknown_pixbuf);
-}
-
-
 static void
 gfv_set_image_text (GthFileView  *file_view,
 		    int           pos,
@@ -366,7 +340,7 @@ gfv_get_file_list_selection (GthFileView *file_view)
 		FileData *fd = scan->data;
 
 		if ((fd != NULL) && (fd->path != NULL))
-			file_list = g_list_prepend (file_list, 
+			file_list = g_list_prepend (file_list,
 						    g_strdup (fd->path));
 	}
 	file_data_list_free (list);
@@ -386,36 +360,6 @@ gfv_pos_is_selected (GthFileView     *file_view,
 }
 
 
-static gboolean
-gfv_only_one_is_selected (GthFileView    *file_view)
-{
-	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
-	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-        GList             *sel_list = gth_image_list_get_selection (ilist);
-	gboolean           ret_val = FALSE;
-
-	ret_val = (sel_list != NULL) && (sel_list->next == NULL);
-	file_data_list_free (sel_list);
-
-	return ret_val;
-}
-
-
-static gboolean
-gfv_selection_not_null (GthFileView    *file_view)
-{
-	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
-	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-        GList             *sel_list = gth_image_list_get_selection (ilist);
-	gboolean           ret_val;
-
-	ret_val = (sel_list != NULL);
-	file_data_list_free (sel_list);
-
-	return ret_val;
-}
-
-
 static int
 gfv_get_first_selected (GthFileView *file_view)
 {
@@ -436,6 +380,16 @@ gfv_get_last_selected (GthFileView *file_view)
 }
 
 
+static int
+gfv_get_n_selected (GthFileView    *file_view)
+{
+	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
+	GthImageList      *ilist = gfv_thumbs->priv->ilist;
+	
+	return gth_image_list_get_n_selected (ilist);
+}
+
+
 /* Setting spacing values */
 
 
@@ -455,7 +409,7 @@ gfv_set_image_width (GthFileView     *file_view,
 
 static void
 gfv_set_image_data (GthFileView     *file_view,
-		    int              pos, 
+		    int              pos,
 		    gpointer         data)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
@@ -465,19 +419,6 @@ gfv_set_image_data (GthFileView     *file_view,
 }
 
 
-static void
-gfv_set_image_data_full (GthFileView     *file_view,
-			 int              pos, 
-			 gpointer         data,
-			 GtkDestroyNotify destroy)
-{
-	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
-	GthImageList      *ilist = gfv_thumbs->priv->ilist;
-
-	gth_image_list_set_image_data_full (ilist, pos, data, destroy);
-}
-
-
 static int
 gfv_find_image_from_data (GthFileView     *file_view,
 			  gpointer         data)
@@ -537,7 +478,7 @@ gfv_get_view_mode (GthFileView *file_view)
 
 static void
 gfv_moveto (GthFileView *file_view,
-	    int          pos, 
+	    int          pos,
 	    double       yalign)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
@@ -559,8 +500,8 @@ gfv_image_is_visible (GthFileView *file_view,
 
 
 static int
-gfv_get_image_at (GthFileView *file_view, 
-		  int          x, 
+gfv_get_image_at (GthFileView *file_view,
+		  int          x,
 		  int          y)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
@@ -590,6 +531,18 @@ gfv_get_last_visible (GthFileView *file_view)
 }
 
 
+static void
+gfv_set_visible_func (GthFileView    *file_view,
+                      GthVisibleFunc  func,
+                      gpointer        data)
+{
+	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
+	GthImageList      *ilist = gfv_thumbs->priv->ilist;
+
+	gth_image_list_set_visible_func (ilist, func, data);
+}
+
+
 /* Sort */
 
 
@@ -606,7 +559,7 @@ comp_func_name (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return strcasecmp (fd1->name, fd2->name);
+	return gth_sort_by_filename_but_ignore_path (fd1->name, fd2->name);
 }
 
 
@@ -623,11 +576,8 @@ comp_func_size (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->size < fd2->size) return -1;
-	if (fd1->size > fd2->size) return 1;
-
-	/* if the size is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+	return gth_sort_by_size_then_name (fd1->size, fd2->size, 
+					   fd1->path, fd2->path);
 }
 
 
@@ -644,11 +594,25 @@ comp_func_time (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->mtime < fd2->mtime) return -1;
-	if (fd1->mtime > fd2->mtime) return 1;
+	return gth_sort_by_filetime_then_name (fd1->mtime, fd2->mtime,
+					       fd1->path, fd2->path);
+}
 
-	/* if time is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+
+static int
+comp_func_exif_date (gconstpointer  ptr1,
+		     gconstpointer  ptr2)
+{
+	const GthImageListItem *item1 = ptr1, *item2 = ptr2;
+	FileData               *fd1, *fd2;
+
+	fd1 = item1->data;
+	fd2 = item2->data;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_exiftime_then_name (fd1, fd2);
 }
 
 
@@ -665,15 +629,21 @@ comp_func_path (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return uricmp (fd1->path, fd2->path);
+	return gth_sort_by_full_path (fd1->path, fd2->path);
 }
 
 
 static int
-comp_func_none (gconstpointer  ptr1,
-		gconstpointer  ptr2)
+comp_func_comment (gconstpointer  ptr1, gconstpointer  ptr2)
 {
-	return 0;
+	const GthImageListItem *item1 = ptr1, *item2 = ptr2;
+	const FileData         *fd1, *fd2;
+
+	fd1 = item1->data;
+	fd2 = item2->data;
+
+	return gth_sort_by_comment_then_name (item1->comment, item2->comment,
+					fd1->path, fd2->path);
 }
 
 
@@ -695,10 +665,16 @@ get_compfunc_from_method (GthSortMethod sort_method)
 	case GTH_SORT_METHOD_BY_PATH:
 		func = comp_func_path;
 		break;
+	case GTH_SORT_METHOD_BY_COMMENT:
+		func = comp_func_comment;
+		break;
+	case GTH_SORT_METHOD_BY_EXIF_DATE:
+		func = comp_func_exif_date;
+		break;
 	case GTH_SORT_METHOD_NONE:
 	case GTH_SORT_METHOD_MANUAL:
 	default:
-		func = comp_func_none;
+		func = gth_sort_none;
 		break;
 	}
 
@@ -734,7 +710,7 @@ gfv_unsorted (GthFileView *file_view)
 
 
 static void
-gfv_image_activated (GthFileView *file_view, 
+gfv_image_activated (GthFileView *file_view,
 		     int          pos)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
@@ -745,7 +721,7 @@ gfv_image_activated (GthFileView *file_view,
 
 
 static void
-gfv_set_cursor (GthFileView *file_view, 
+gfv_set_cursor (GthFileView *file_view,
 		int          pos)
 {
 	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
@@ -765,71 +741,6 @@ gfv_get_cursor (GthFileView *file_view)
 }
 
 
-static int
-get_default_icon_size (GtkWidget *widget)
-{
-	int icon_width, icon_height;
-
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           GTK_ICON_SIZE_DIALOG,
-                                           &icon_width, &icon_height);
-	return MAX (icon_width, icon_height);
-}
-
-
-static GdkPixbuf *
-create_unknown_pixbuf (GthFileViewThumbs *gfv_thumbs)
-{
-	int         icon_size;
-	char       *icon_name;
-	char       *icon_path;
-	GdkPixbuf  *pixbuf = NULL;
-	int         width, height;
-
-	icon_size = get_default_icon_size (GTK_WIDGET (gfv_thumbs->priv->ilist));
-
-	icon_name = gnome_icon_lookup (gfv_thumbs->priv->icon_theme,
-				       NULL,
-				       NULL,
-				       NULL,
-				       NULL,
-				       "image/*",
-				       GNOME_ICON_LOOKUP_FLAGS_NONE,
-				       NULL);
-	icon_path = gnome_icon_theme_lookup_icon (gfv_thumbs->priv->icon_theme,
-						  icon_name,
-						  icon_size,
-						  NULL,
-						  NULL);
-	g_free (icon_name);
-
-	if (icon_path != NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
-		g_free (icon_path);
-	}
-
-	if (pixbuf == NULL)
-		pixbuf = gdk_pixbuf_new_from_inline (-1, 
-						     dir_16_rgba, 
-						     FALSE, 
-						     NULL);
-
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	if (scale_keepping_ratio (&width, &height, icon_size, icon_size)) {
-		GdkPixbuf *scaled;
-		scaled = gdk_pixbuf_scale_simple (pixbuf,
-						  width,
-						  height,
-						  GDK_INTERP_BILINEAR);
-		g_object_unref (pixbuf);
-		pixbuf = scaled;
-	}
-
-	return pixbuf;
-}
-
-
 static void
 gth_file_view_thumbs_init (GthFileViewThumbs *gfv_thumbs)
 {
@@ -840,18 +751,6 @@ gth_file_view_thumbs_init (GthFileViewThumbs *gfv_thumbs)
 }
 
 
-static void
-gfv_update_icon_theme (GthFileView *file_view)
-{
-	GthFileViewThumbs *gfv_thumbs = (GthFileViewThumbs *) file_view;
-
-	if (gfv_thumbs->priv->unknown_pixbuf != NULL) 
-		g_object_unref (gfv_thumbs->priv->unknown_pixbuf);
-
-	gfv_thumbs->priv->unknown_pixbuf = create_unknown_pixbuf (gfv_thumbs);
-}
-
-
 static void
 gfv_set_no_image_text (GthFileView *file_view,
 		       const char  *text)
@@ -946,8 +845,6 @@ gth_file_view_thumbs_finalize (GObject *object)
 
 	gfv_thumbs = (GthFileViewThumbs*) object;
 
-	g_object_unref (gfv_thumbs->priv->icon_theme);
-	g_object_unref (gfv_thumbs->priv->unknown_pixbuf);
 	g_free (gfv_thumbs->priv);
 
         /* Chain up */
@@ -983,7 +880,6 @@ gth_file_view_thumbs_class_init (GthFileViewThumbsClass *file_view_thumbs_class)
 	file_view_class->remove               = gfv_remove;
 	file_view_class->clear                = gfv_clear;
 	file_view_class->set_image_pixbuf     = gfv_set_image_pixbuf;
-	file_view_class->set_unknown_pixbuf     = gfv_set_unknown_pixbuf;
 	file_view_class->set_image_text       = gfv_set_image_text;
 	file_view_class->get_image_text       = gfv_get_image_text;
 	file_view_class->set_image_comment    = gfv_set_image_comment;
@@ -997,13 +893,11 @@ gth_file_view_thumbs_class_init (GthFileViewThumbsClass *file_view_thumbs_class)
 	file_view_class->unselect_all         = gfv_unselect_all;
 	file_view_class->get_file_list_selection = gfv_get_file_list_selection;
 	file_view_class->pos_is_selected      = gfv_pos_is_selected;
-	file_view_class->only_one_is_selected = gfv_only_one_is_selected;
-	file_view_class->selection_not_null   = gfv_selection_not_null;
 	file_view_class->get_first_selected   = gfv_get_first_selected;
 	file_view_class->get_last_selected    = gfv_get_last_selected;
+	file_view_class->get_n_selected       = gfv_get_n_selected;	
 	file_view_class->set_image_width      = gfv_set_image_width;
 	file_view_class->set_image_data       = gfv_set_image_data;
-	file_view_class->set_image_data_full  = gfv_set_image_data_full;
 	file_view_class->find_image_from_data = gfv_find_image_from_data;
 	file_view_class->get_image_data       = gfv_get_image_data;
 	file_view_class->enable_thumbs        = gfv_enable_thumbs;
@@ -1014,12 +908,12 @@ gth_file_view_thumbs_class_init (GthFileViewThumbsClass *file_view_thumbs_class)
 	file_view_class->get_image_at         = gfv_get_image_at;
 	file_view_class->get_first_visible    = gfv_get_first_visible;
 	file_view_class->get_last_visible     = gfv_get_last_visible;
+	file_view_class->set_visible_func     = gfv_set_visible_func;
 	file_view_class->sorted               = gfv_sorted;
 	file_view_class->unsorted             = gfv_unsorted;
 	file_view_class->image_activated      = gfv_image_activated;
 	file_view_class->set_cursor           = gfv_set_cursor;
 	file_view_class->get_cursor           = gfv_get_cursor;
-	file_view_class->update_icon_theme    = gfv_update_icon_theme;
 	file_view_class->set_no_image_text    = gfv_set_no_image_text;
 	file_view_class->set_drag_dest_pos    = gfv_set_drag_dest_pos;
 	file_view_class->get_drag_dest_pos    = gfv_get_drag_dest_pos;
@@ -1058,7 +952,7 @@ gth_file_view_thumbs_get_type (void)
 }
 
 
-static void 
+static void
 selection_changed_cb (GthImageList      *image_list,
 		      GthFileViewThumbs *gfv_thumbs)
 {
@@ -1091,7 +985,7 @@ gth_file_view_thumbs_new (guint image_width)
 
 	gfv_thumbs = GTH_FILE_VIEW_THUMBS (g_object_new (GTH_TYPE_FILE_VIEW_THUMBS, NULL));
 
-	gfv_thumbs->priv->ilist = (GthImageList *) gth_image_list_new (image_width);
+	gfv_thumbs->priv->ilist = (GthImageList *) gth_image_list_new (image_width, GTH_TYPE_FILE_DATA);
 	g_signal_connect (G_OBJECT (gfv_thumbs->priv->ilist),
 			  "selection_changed",
 			  G_CALLBACK (selection_changed_cb),
@@ -1105,11 +999,5 @@ gth_file_view_thumbs_new (guint image_width)
 			  G_CALLBACK (cursor_changed_cb),
 			  gfv_thumbs);
 
-	/**/
-
-	gfv_thumbs->priv->icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (gfv_thumbs->priv->icon_theme, TRUE);
-	gfv_thumbs->priv->unknown_pixbuf = create_unknown_pixbuf (gfv_thumbs);
-
 	return GTH_FILE_VIEW (gfv_thumbs);
 }
