@@ -48,7 +48,7 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* BOOL:BOOL (../libgthumb/gthumb-marshal.list:1) */
+/* BOOL:BOOL (./gthumb-marshal.list:1) */
 void
 gthumb_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                  GValue       *return_value,
@@ -87,7 +87,7 @@ gthumb_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:ENUM (../libgthumb/gthumb-marshal.list:2) */
+/* BOOL:ENUM (./gthumb-marshal.list:2) */
 void
 gthumb_marshal_BOOLEAN__ENUM (GClosure     *closure,
                               GValue       *return_value,
@@ -126,7 +126,7 @@ gthumb_marshal_BOOLEAN__ENUM (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:ENUM,ENUM (../libgthumb/gthumb-marshal.list:3) */
+/* BOOL:ENUM,ENUM (./gthumb-marshal.list:3) */
 void
 gthumb_marshal_BOOLEAN__ENUM_ENUM (GClosure     *closure,
                                    GValue       *return_value,
@@ -167,7 +167,7 @@ gthumb_marshal_BOOLEAN__ENUM_ENUM (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:INT,POINTER (../libgthumb/gthumb-marshal.list:4) */
+/* BOOL:INT,POINTER (./gthumb-marshal.list:4) */
 void
 gthumb_marshal_BOOLEAN__INT_POINTER (GClosure     *closure,
                                      GValue       *return_value,
@@ -208,7 +208,7 @@ gthumb_marshal_BOOLEAN__INT_POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:VOID (../libgthumb/gthumb-marshal.list:5) */
+/* BOOL:VOID (./gthumb-marshal.list:5) */
 void
 gthumb_marshal_BOOLEAN__VOID (GClosure     *closure,
                               GValue       *return_value,
@@ -245,9 +245,9 @@ gthumb_marshal_BOOLEAN__VOID (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:BOOL (../libgthumb/gthumb-marshal.list:6) */
+/* VOID:BOOL (./gthumb-marshal.list:6) */
 
-/* VOID:ENUM,BOOLEAN (../libgthumb/gthumb-marshal.list:7) */
+/* VOID:ENUM,BOOLEAN (./gthumb-marshal.list:7) */
 void
 gthumb_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                    GValue       *return_value,
@@ -284,7 +284,7 @@ gthumb_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM,ENUM (../libgthumb/gthumb-marshal.list:8) */
+/* VOID:ENUM,ENUM (./gthumb-marshal.list:8) */
 void
 gthumb_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                 GValue       *return_value,
@@ -321,9 +321,13 @@ gthumb_marshal_VOID__ENUM_ENUM (GClosure     *closure,
             data2);
 }
 
-/* VOID:FLOAT (../libgthumb/gthumb-marshal.list:9) */
+/* VOID:ENUM (./gthumb-marshal.list:9) */
 
-/* VOID:INT,BOXED (../libgthumb/gthumb-marshal.list:10) */
+/* VOID:FLOAT (./gthumb-marshal.list:10) */
+
+/* VOID:INT (./gthumb-marshal.list:11) */
+
+/* VOID:INT,BOXED (./gthumb-marshal.list:12) */
 void
 gthumb_marshal_VOID__INT_BOXED (GClosure     *closure,
                                 GValue       *return_value,
@@ -360,7 +364,44 @@ gthumb_marshal_VOID__INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (../libgthumb/gthumb-marshal.list:11) */
+/* VOID:INT,INT (./gthumb-marshal.list:13) */
+void
+gthumb_marshal_VOID__INT_INT (GClosure     *closure,
+                              GValue       *return_value,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint,
+                              gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_INT) (gpointer     data1,
+                                              gint         arg_1,
+                                              gint         arg_2,
+                                              gpointer     data2);
+  register GMarshalFunc_VOID__INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            data2);
+}
+
+/* VOID:POINTER,POINTER (./gthumb-marshal.list:14) */
 void
 gthumb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                       GValue       *return_value,
@@ -397,9 +438,9 @@ gthumb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING (../libgthumb/gthumb-marshal.list:12) */
+/* VOID:STRING (./gthumb-marshal.list:15) */
 
-/* VOID:STRING,INT (../libgthumb/gthumb-marshal.list:13) */
+/* VOID:STRING,INT (./gthumb-marshal.list:16) */
 void
 gthumb_marshal_VOID__STRING_INT (GClosure     *closure,
                                  GValue       *return_value,
@@ -436,7 +477,7 @@ gthumb_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT,BOXED (../libgthumb/gthumb-marshal.list:14) */
+/* VOID:STRING,INT,BOXED (./gthumb-marshal.list:17) */
 void
 gthumb_marshal_VOID__STRING_INT_BOXED (GClosure     *closure,
                                        GValue       *return_value,
@@ -475,7 +516,7 @@ gthumb_marshal_VOID__STRING_INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (../libgthumb/gthumb-marshal.list:15) */
+/* VOID:STRING,STRING (./gthumb-marshal.list:18) */
 void
 gthumb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                     GValue       *return_value,
@@ -512,5 +553,5 @@ gthumb_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:VOID (../libgthumb/gthumb-marshal.list:16) */
+/* VOID:VOID (./gthumb-marshal.list:19) */
 
