@@ -42,11 +42,11 @@
  */
 
 /* The jpeg saver is based upon the gdk-pixbuf library, which has the
- * following copyright note : 
+ * following copyright note :
  *
  * Copyright (C) 1999 Michael Zucchi
  * Copyright (C) 1999 The Free Software Foundation
- * 
+ *
  * Progressive loading code Copyright (C) 1999 Red Hat, Inc.
  *
  * Authors: Michael Zucchi <zucchi@zedzone.mmc.com.au>
@@ -86,187 +86,60 @@
 #include <jpeglib.h>
 
 
-enum {
-	RED_PIX   = 0,
-	GREEN_PIX = 1,
-	BLUE_PIX  = 2,
-	ALPHA_PIX = 3
-};
-
-
-/*
- * Returns a copy of pixbuf src rotated 90 degrees clockwise or 90 
- * counterclockwise.
- */
-GdkPixbuf *
-_gdk_pixbuf_copy_rotate_90 (GdkPixbuf *src, 
-			    gboolean counter_clockwise)
-{
-	GdkPixbuf *dest;
-	int        has_alpha;
-	int        sw, sh, srs;
-	int        dw, dh, drs;
-	guchar    *s_pix;
-        guchar    *d_pix;
-	guchar    *sp;
-        guchar    *dp;
-	int        i, j;
-	int        a;
-
-	if (!src) return NULL;
-
-	sw = gdk_pixbuf_get_width (src);
-	sh = gdk_pixbuf_get_height (src);
-	has_alpha = gdk_pixbuf_get_has_alpha (src);
-	srs = gdk_pixbuf_get_rowstride (src);
-	s_pix = gdk_pixbuf_get_pixels (src);
-
-	dw = sh;
-	dh = sw;
-	dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB, has_alpha, 8, dw, dh);
-	drs = gdk_pixbuf_get_rowstride (dest);
-	d_pix = gdk_pixbuf_get_pixels (dest);
-
-	a = (has_alpha ? 4 : 3);
-
-	for (i = 0; i < sh; i++) {
-		sp = s_pix + (i * srs);
-		for (j = 0; j < sw; j++) {
-			if (counter_clockwise)
-				dp = d_pix + ((dh - j - 1) * drs) + (i * a);
-			else
-				dp = d_pix + (j * drs) + ((dw - i - 1) * a);
-
-			*(dp++) = *(sp++);	/* r */
-			*(dp++) = *(sp++);	/* g */
-			*(dp++) = *(sp++);	/* b */
-			if (has_alpha) *(dp) = *(sp++);	/* a */
-		}
-	}
-
-	return dest;
-}
-
-
-/*
- * Returns a copy of pixbuf mirrored and or flipped.
- * TO do a 180 degree rotations set both mirror and flipped TRUE
- * if mirror and flip are FALSE, result is a simple copy.
- */
-GdkPixbuf *
-_gdk_pixbuf_copy_mirror (GdkPixbuf *src, 
-			 gboolean mirror, 
-			 gboolean flip)
-{
-	GdkPixbuf *dest;
-	int        has_alpha;
-	int        w, h, srs;
-	int        drs;
-	guchar    *s_pix;
-        guchar    *d_pix;
-	guchar    *sp;
-        guchar    *dp;
-	int        i, j;
-	int        a;
-
-	if (!src) return NULL;
-
-	w = gdk_pixbuf_get_width (src);
-	h = gdk_pixbuf_get_height (src);
-	has_alpha = gdk_pixbuf_get_has_alpha (src);
-	srs = gdk_pixbuf_get_rowstride (src);
-	s_pix = gdk_pixbuf_get_pixels (src);
-
-	dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB, has_alpha, 8, w, h);
-	drs = gdk_pixbuf_get_rowstride (dest);
-	d_pix = gdk_pixbuf_get_pixels (dest);
-
-	a = has_alpha ? 4 : 3;
-
-	for (i = 0; i < h; i++)	{
-		sp = s_pix + (i * srs);
-		if (flip)
-			dp = d_pix + ((h - i - 1) * drs);
-		else
-			dp = d_pix + (i * drs);
-
-		if (mirror) {
-			dp += (w - 1) * a;
-			for (j = 0; j < w; j++) {
-				*(dp++) = *(sp++);	/* r */
-				*(dp++) = *(sp++);	/* g */
-				*(dp++) = *(sp++);	/* b */
-				if (has_alpha) *(dp) = *(sp++);	/* a */
-				dp -= (a + 3);
-			}
-		} else {
-			for (j = 0; j < w; j++) {
-				*(dp++) = *(sp++);	/* r */
-				*(dp++) = *(sp++);	/* g */
-				*(dp++) = *(sp++);	/* b */
-				if (has_alpha) *(dp++) = *(sp++);	/* a */
-			}
-		}
-	}
-	
-	return dest;
-}
-
-
 /*
  * Returns a transformed image.
  */
 GdkPixbuf *
-_gdk_pixbuf_transform (GdkPixbuf* src, 
+_gdk_pixbuf_transform (GdkPixbuf* src,
 				GthTransform transform)
 {
 	GdkPixbuf *temp = NULL, *dest = NULL;
-	
+
 	if (!src) return NULL;
-	
+
 	switch (transform) {
 	case GTH_TRANSFORM_NONE:
 		dest = src;
 		g_object_ref (dest);
 		break;
 	case GTH_TRANSFORM_FLIP_H:
-		dest = _gdk_pixbuf_copy_mirror (src, TRUE, FALSE);
+		dest = gdk_pixbuf_flip (src, TRUE);
 		break;
 	case GTH_TRANSFORM_ROTATE_180:
-		dest = _gdk_pixbuf_copy_mirror (src, TRUE, TRUE);
+		dest = gdk_pixbuf_rotate_simple (src, GDK_PIXBUF_ROTATE_UPSIDEDOWN);
 		break;
 	case GTH_TRANSFORM_FLIP_V:
-		dest = _gdk_pixbuf_copy_mirror (src, FALSE, TRUE);
+		dest = gdk_pixbuf_flip (src, FALSE);
 		break;
 	case GTH_TRANSFORM_TRANSPOSE:
-		temp = _gdk_pixbuf_copy_rotate_90 (src, FALSE);
-		dest = _gdk_pixbuf_copy_mirror (temp, TRUE, FALSE);
+		temp = gdk_pixbuf_rotate_simple (src, GDK_PIXBUF_ROTATE_CLOCKWISE);
+		dest = gdk_pixbuf_flip (temp, TRUE);
 		g_object_unref (temp);
 		break;
 	case GTH_TRANSFORM_ROTATE_90:
-		dest = _gdk_pixbuf_copy_rotate_90 (src, FALSE);
+		dest = gdk_pixbuf_rotate_simple (src, GDK_PIXBUF_ROTATE_CLOCKWISE);
 		break;
 	case GTH_TRANSFORM_TRANSVERSE:
-		temp = _gdk_pixbuf_copy_rotate_90 (src, FALSE);
-		dest = _gdk_pixbuf_copy_mirror (temp, FALSE, TRUE);
+		temp = gdk_pixbuf_rotate_simple (src, GDK_PIXBUF_ROTATE_CLOCKWISE);
+		dest = gdk_pixbuf_flip (temp, FALSE);
 		g_object_unref (temp);
 		break;
 	case GTH_TRANSFORM_ROTATE_270:
-		dest = _gdk_pixbuf_copy_rotate_90 (src, TRUE);
+		dest = gdk_pixbuf_rotate_simple (src, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
 		break;
 	default:
 		dest = src;
 		g_object_ref (dest);
 		break;
 	}
-	
+
 	return dest;
 }
 
 
 void
-pixmap_from_xpm (const char **data, 
-		 GdkPixmap **pixmap, 
+pixmap_from_xpm (const char **data,
+		 GdkPixmap **pixmap,
 		 GdkBitmap **mask)
 {
 	GdkPixbuf *pixbuf;
@@ -278,7 +151,7 @@ pixmap_from_xpm (const char **data,
 
 
 void
-_gdk_pixbuf_vertical_gradient (GdkPixbuf *pixbuf, 
+_gdk_pixbuf_vertical_gradient (GdkPixbuf *pixbuf,
 			       guint32    color1,
 			       guint32    color2)
 {
@@ -292,7 +165,7 @@ _gdk_pixbuf_vertical_gradient (GdkPixbuf *pixbuf,
 	guint     width, height;
 	guint     w, h;
 	int       n_channels, rowstride;
-	
+
 	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
 
 	width = gdk_pixbuf_get_width (pixbuf);
@@ -368,7 +241,7 @@ _gdk_pixbuf_vertical_gradient (GdkPixbuf *pixbuf,
 
 
 void
-_gdk_pixbuf_horizontal_gradient (GdkPixbuf *pixbuf, 
+_gdk_pixbuf_horizontal_gradient (GdkPixbuf *pixbuf,
 				 guint32    color1,
 				 guint32    color2)
 {
@@ -382,7 +255,7 @@ _gdk_pixbuf_horizontal_gradient (GdkPixbuf *pixbuf,
 	guint     width, height;
 	guint     w, h;
 	int       n_channels, rowstride;
-	
+
 	g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
 
 	width = gdk_pixbuf_get_width (pixbuf);
@@ -458,7 +331,7 @@ _gdk_pixbuf_horizontal_gradient (GdkPixbuf *pixbuf,
 
 
 void
-_gdk_pixbuf_hv_gradient (GdkPixbuf *pixbuf, 
+_gdk_pixbuf_hv_gradient (GdkPixbuf *pixbuf,
 			 guint32    hcolor1,
 			 guint32    hcolor2,
 			 guint32    vcolor1,
@@ -559,12 +432,51 @@ _gdk_pixbuf_hv_gradient (GdkPixbuf *pixbuf,
 }
 
 
+GdkPixbuf*          
+_gdk_pixbuf_scale_simple_safe (const GdkPixbuf *src,
+                               int              dest_width,
+                               int              dest_height,
+                               GdkInterpType    interp_type)
+{
+	GdkPixbuf* temp_pixbuf1;
+	GdkPixbuf* temp_pixbuf2;
+	int        x_ratio, y_ratio;
+	int        temp_width = dest_width, temp_height = dest_height;
+
+	g_assert (dest_width > 1);
+	g_assert (dest_height > 1);
+
+	x_ratio = gdk_pixbuf_get_width (src) / dest_width;
+	y_ratio = gdk_pixbuf_get_height (src) / dest_height;
+
+	/* The gdk_pixbuf scaling routines do not handle large-ratio downscaling
+	   very well. Memory usage explodes and the application may freeze or crash.
+	   For scale-down ratios in excess of 100, do the scale in two steps.
+	   It is faster and safer that way. See bug 80925 for background info. */
+
+	if (x_ratio > 100)
+		/* Scale down to 10x the requested size first. */
+		temp_width = 10 * dest_width;
+
+	if (y_ratio > 100)
+		/* Scale down to 10x the requested size first. */
+		temp_height = 10 * dest_height;
+
+	if ( (temp_width != dest_width) || (temp_height != dest_height)) {
+		temp_pixbuf1 = gdk_pixbuf_scale_simple (src, temp_width, temp_height, interp_type);
+		temp_pixbuf2 = gdk_pixbuf_scale_simple (temp_pixbuf1, dest_width, dest_height, interp_type);
+		g_object_unref (temp_pixbuf1);
+	} else
+		temp_pixbuf2 = gdk_pixbuf_scale_simple (src, dest_width, dest_height, interp_type);
+
+	return temp_pixbuf2;
+}
 
 /* error handler data */
 struct error_handler_data {
-	struct jpeg_error_mgr pub;
-	sigjmp_buf setjmp_buffer;
-        GError **error;
+	struct jpeg_error_mgr   pub;
+	sigjmp_buf              setjmp_buffer;
+        GError                **error;
 };
 
 
@@ -573,9 +485,9 @@ fatal_error_handler (j_common_ptr cinfo)
 {
 	struct error_handler_data *errmgr;
         char buffer[JMSG_LENGTH_MAX];
-        
+
 	errmgr = (struct error_handler_data *) cinfo->err;
-        
+
         /* Create the message */
         (* cinfo->err->format_message) (cinfo, buffer);
 
@@ -589,7 +501,7 @@ fatal_error_handler (j_common_ptr cinfo)
 			     "Error interpreting JPEG image file (%s)",
                              buffer);
         }
-        
+
 	siglongjmp (errmgr->setjmp_buffer, 1);
 
         g_assert_not_reached ();
@@ -605,7 +517,7 @@ output_message_handler (j_common_ptr cinfo)
 
 
 static gboolean
-_gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf, 
+_gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 			  const char    *filename,
 			  char         **keys,
 			  char         **values,
@@ -629,7 +541,7 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 	if (keys && *keys) {
 		char **kiter = keys;
 		char **viter = values;
-		
+
 		while (*kiter) {
 			if (strcmp (*kiter, "quality") == 0) {
 				char *endptr = NULL;
@@ -641,17 +553,17 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG quality must be a value between 0 and 100; value '%s' could not be parsed.",
 						     *viter);
-					
+
 					return FALSE;
 				}
-				
+
 				if (quality < 0 || quality > 100) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG quality must be a value between 0 and 100; value '%d' is not allowed.",
 						     quality);
-					
+
 					return FALSE;
 				}
 
@@ -665,17 +577,17 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG smoothing must be a value between 0 and 100; value '%s' could not be parsed.",
 						     *viter);
-					
+
 					return FALSE;
 				}
-				
+
 				if (smoothing < 0 || smoothing > 100) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG smoothing must be a value between 0 and 100; value '%d' is not allowed.",
 						     smoothing);
-					
+
 					return FALSE;
 				}
 
@@ -689,7 +601,7 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG optimize option must be 'yes' or 'no', value is: %s", *viter);
-					
+
 					return FALSE;
 				}
 
@@ -703,33 +615,33 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "JPEG progressive option must be 'yes' or 'no', value is: %s", *viter);
-					
+
 					return FALSE;
 				}
-				
+
 			} else {
 				g_warning ("Bad option name '%s' passed to JPEG saver", *kiter);
 				return FALSE;
 			}
-			
+
 			++kiter;
 			++viter;
 		}
 	}
-	
+
 	file = fopen (filename, "wb");
-	
+
 	if (file == NULL) {
 		g_set_error (error,
 			     GDK_PIXBUF_ERROR,
 			     GDK_PIXBUF_ERROR_FAILED,
-			     "Can't write image to file '%s'", 
+			     "Can't write image to file '%s'",
 			     filename);
 		return FALSE;
 	}
-       
+
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	
+
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
 
@@ -737,11 +649,11 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 		bpp = 4;
 	else
 		bpp = 3;
-	
+
 	/* no image data? abort */
 	pixels = gdk_pixbuf_get_pixels (pixbuf);
 	g_return_val_if_fail (pixels != NULL, FALSE);
-	
+
 	/* allocate a small buffer to convert image data */
 	buf = g_try_malloc (w * bpp * sizeof (guchar));
 	if (! buf) {
@@ -769,9 +681,9 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 	jpeg_stdio_dest (&cinfo, file);
 	cinfo.image_width      = w;
 	cinfo.image_height     = h;
-	cinfo.input_components = 3; 
+	cinfo.input_components = 3;
 	cinfo.in_color_space   = JCS_RGB;
-	
+
 	/* set up jepg compression parameters */
 	jpeg_set_defaults (&cinfo);
 	jpeg_set_quality (&cinfo, quality, TRUE);
@@ -779,7 +691,7 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 	cinfo.optimize_coding = optimize;
 
 #ifdef HAVE_PROGRESSIVE_JPEG
-	if (progressive) 
+	if (progressive)
 		jpeg_simple_progression (&cinfo);
 #endif /* HAVE_PROGRESSIVE_JPEG */
 
@@ -793,23 +705,23 @@ _gdk_pixbuf_save_as_jpeg (GdkPixbuf     *pixbuf,
 
 		/* convert scanline from ARGB to RGB packed */
 		for (j = 0; j < w; j++)
-			memcpy (&(buf[j * 3]), 
-				&(ptr[i * rowstride + j * bpp]), 
+			memcpy (&(buf[j * 3]),
+				&(ptr[i * rowstride + j * bpp]),
 				3);
-		
+
 		/* write scanline */
 		jbuf = (JSAMPROW *)(&buf);
 		jpeg_write_scanlines (&cinfo, jbuf, 1);
 		i++;
 	}
-	
+
 	/* finish off */
 	jpeg_finish_compress (&cinfo);
 	fclose (file);
 
 	jpeg_destroy_compress(&cinfo);
 	g_free (buf);
-	
+
 	return TRUE;
 }
 
@@ -847,13 +759,13 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 	if (keys && *keys) {
 		char **kiter = keys;
 		char **viter = values;
-		
+
 		while (*kiter) {
 			if (strcmp (*kiter, "compression") == 0) {
 				if (*viter == NULL) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
-						     GDK_PIXBUF_ERROR_BAD_OPTION, 
+						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "Must specify a compression type");
 					return FALSE;
 				}
@@ -871,11 +783,11 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 				else {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
-						     GDK_PIXBUF_ERROR_BAD_OPTION,       
+						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "Unsupported compression type passed to the TIFF saver");
 					return FALSE;
 				}
-				
+
 			} else if (strcmp (*kiter, "vertical dpi") == 0) {
 				char *endptr = NULL;
 				vertical_dpi = strtol (*viter, &endptr, 10);
@@ -887,54 +799,54 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "TIFF vertical dpi must be a value greater than 0; value '%s' could not be parsed.",
 						     *viter);
-					
+
 					return FALSE;
 				}
-				
+
 				if (vertical_dpi < 0) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "TIFF vertical dpi must be a value greater than 0; value '%d' is not allowed.",
 						     vertical_dpi);
-					
+
 					return FALSE;
 				}
-				
+
 			} else if (strcmp (*kiter, "horizontal dpi") == 0) {
 				char *endptr = NULL;
 				horizontal_dpi = strtol (*viter, &endptr, 10);
 				save_resolution = TRUE;
-				
+
 				if (endptr == *viter) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "TIFF horizontal dpi must be a value greater than 0; value '%s' could not be parsed.",
 						     *viter);
-					
+
 					return FALSE;
 				}
-				
+
 				if (horizontal_dpi < 0) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
 						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "TIFF horizontal dpi must be a value greater than 0; value '%d' is not allowed.",
 						     horizontal_dpi);
-					
+
 					return FALSE;
 				}
 			} else {
 				g_warning ("Bad option name '%s' passed to the TIFF saver", *kiter);
 				return FALSE;
 			}
-			
+
 			++kiter;
 			++viter;
 		}
 	}
-				
+
 	predictor    = 0;
 	rowsperstrip = TILE_HEIGHT;
 
@@ -944,7 +856,7 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 		g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
-			     "Can't write image to file '%s'", 
+			     "Can't write image to file '%s'",
 			     filename);
 		return FALSE;
 	}
@@ -963,7 +875,7 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 		samplesperpixel = 4;
 	else
 		samplesperpixel = 3;
-	
+
 	/* Set TIFF parameters. */
 
 	TIFFSetField (tif, TIFFTAG_SUBFILETYPE,   0);
@@ -1016,9 +928,9 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 		/* convert scanline from ARGB to RGB packed */
 		for (col = 0; col < cols; col++)
 			memcpy (&(buf[col * 3]), &(ptr[col * samplesperpixel /*3*/]), 3);
-		
+
 		success = TIFFWriteScanline (tif, buf, row, 0) >= 0;
-		
+
 		if (! success) {
 			g_set_error (error,
 				     GDK_PIXBUF_ERROR,
@@ -1030,21 +942,18 @@ _gdk_pixbuf_save_as_tiff (GdkPixbuf   *pixbuf,
 
 		ptr += rowstride;
 	}
-	
+
 	TIFFFlushData (tif);
 	TIFFClose (tif);
 
 	g_free (buf);
-	
+
 	return TRUE;
 }
 
 #endif
 
 
-
-
-
 /* TRUEVISION-XFILE magic signature string */
 static guchar magic[18] = {
 	0x54, 0x52, 0x55, 0x45, 0x56, 0x49, 0x53, 0x49, 0x4f,
@@ -1062,7 +971,7 @@ rle_write (FILE   *fp,
 	int     direct = 0;
 	guchar *from   = buffer;
 	guint   x;
-	
+
 	for (x = 1; x < width; ++x) {
 		if (memcmp (buffer, buffer + bytes, bytes)) {
 			/* next pixel is different */
@@ -1074,7 +983,7 @@ rle_write (FILE   *fp,
 				direct = 0;
 			} else
 				direct += 1;
-			
+
 		} else {
 			/* next pixel is the same */
 			if (direct) {
@@ -1083,7 +992,7 @@ rle_write (FILE   *fp,
 				from = buffer; /* point to first identical pixel */
 				direct = 0;
 				repeat = 1;
-			} else 
+			} else
 				repeat += 1;
 		}
 
@@ -1100,10 +1009,10 @@ rle_write (FILE   *fp,
 			direct = 0;
 			repeat = 0;
 		}
-		
+
 		buffer += bytes;
 	}
-	
+
 	if (repeat > 0) {
 		putc (128 + repeat, fp);
 		fwrite (from, bytes, 1, fp);
@@ -1122,22 +1031,22 @@ bgr2rgb (guchar *dest,
 	 guint   alpha)
 {
 	guint x;
-	
-	if (alpha) 
+
+	if (alpha)
 		for (x = 0; x < width; x++) {
 			*(dest++) = src[2];
 			*(dest++) = src[1];
 			*(dest++) = src[0];
 			*(dest++) = src[3];
-			
+
 			src += bytes;
 		}
-	else 
+	else
 		for (x = 0; x < width; x++) {
 			*(dest++) = src[2];
 			*(dest++) = src[1];
 			*(dest++) = src[0];
-			
+
 			src += bytes;
 		}
 }
@@ -1166,13 +1075,13 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 	if (keys && *keys) {
 		char **kiter = keys;
 		char **viter = values;
-		
+
 		while (*kiter) {
 			if (strcmp (*kiter, "compression") == 0) {
 				if (*viter == NULL) {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
-						     GDK_PIXBUF_ERROR_BAD_OPTION,   
+						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "Must specify a compression type");
 					return FALSE;
 				}
@@ -1186,7 +1095,7 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 				else {
 					g_set_error (error,
 						     GDK_PIXBUF_ERROR,
-						     GDK_PIXBUF_ERROR_BAD_OPTION,    
+						     GDK_PIXBUF_ERROR_BAD_OPTION,
 						     "Unsupported compression type passed to the TGA saver");
 					return FALSE;
 				}
@@ -1194,7 +1103,7 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 				g_warning ("Bad option name '%s' passed to the TGA saver", *kiter);
 				return FALSE;
 			}
-			
+
 			++kiter;
 			++viter;
 		}
@@ -1210,11 +1119,11 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 		g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_FAILED,
-			     "Can't write image to file '%s'", 
+			     "Can't write image to file '%s'",
 			     filename);
 		return FALSE;
 	}
-	
+
 	header[0] = 0; /* No image identifier / description */
 
 	header[1]= 0;
@@ -1259,7 +1168,7 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 	for (row = 0; row < height; ++row) {
 		bgr2rgb (buf, ptr, width, out_bpp, alpha);
 
-		if (rle_compression) 
+		if (rle_compression)
 			rle_write (fp, buf, width, out_bpp);
 		else
 			fwrite (buf, width * out_bpp, 1, fp);
@@ -1273,9 +1182,9 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 	memset (footer, 0, 8); /* No extensions, no developer directory */
 	memcpy (footer + 8, magic, sizeof (magic)); /* magic signature */
 	fwrite (footer, sizeof (footer), 1, fp);
-	
+
 	fclose (fp);
-	
+
 	return TRUE;
 }
 
@@ -1285,41 +1194,40 @@ _gdk_pixbuf_save_as_tga (GdkPixbuf   *pixbuf,
 
 gboolean
 _gdk_pixbuf_savev (GdkPixbuf    *pixbuf,
-		   const char   *filename,
+		   const char   *local_file,
 		   const char   *type,
 		   char        **keys,
 		   char        **values,
 		   GError      **error)
 {
-	gboolean   result;
+	gboolean result;
 
-	g_return_val_if_fail (pixbuf != NULL, TRUE);
-	g_return_val_if_fail (filename != NULL, TRUE);
-	g_return_val_if_fail (type != NULL, TRUE);
-
-	filename = get_file_path_from_uri (filename);
+	g_return_val_if_fail (pixbuf != NULL, FALSE);
+	g_return_val_if_fail (local_file != NULL, FALSE);
+	g_return_val_if_fail (type != NULL, FALSE);
+	g_return_val_if_fail (! uri_has_scheme (local_file), FALSE);
 
 #ifdef HAVE_LIBTIFF
-	if (strcmp (type, "tiff") == 0) 
-		result = _gdk_pixbuf_save_as_tiff (pixbuf, 
-						   filename, 
-						   keys, values, 
+	if (strcmp (type, "tiff") == 0)
+		result = _gdk_pixbuf_save_as_tiff (pixbuf,
+						   local_file,
+						   keys, values,
 						   error);
 	else
 #endif
-	if (strcmp (type, "jpeg") == 0) 
-		result = _gdk_pixbuf_save_as_jpeg (pixbuf, 
-						   filename, 
-						   keys, values, 
+	if (strcmp (type, "jpeg") == 0)
+		result = _gdk_pixbuf_save_as_jpeg (pixbuf,
+						   local_file,
+						   keys, values,
 						   error);
 	else
 	if ((strcmp (type, "x-tga") == 0) || (strcmp (type, "tga") == 0))
-		result = _gdk_pixbuf_save_as_tga (pixbuf, 
-						  filename, 
-						  keys, values, 
+		result = _gdk_pixbuf_save_as_tga (pixbuf,
+						  local_file,
+						  keys, values,
 						  error);
 	else
-		result = gdk_pixbuf_savev (pixbuf, filename, type,
+		result = gdk_pixbuf_savev (pixbuf, local_file, type,
 					   keys, values,
 					   error);
 
@@ -1340,7 +1248,7 @@ collect_save_options (va_list    opts,
 	count = 0;
 	*keys = NULL;
 	*vals = NULL;
-	
+
 	next = va_arg (opts, gchar*);
 	while (next) {
 		key = next;
@@ -1351,13 +1259,13 @@ collect_save_options (va_list    opts,
 		/* woo, slow */
 		*keys = g_realloc (*keys, sizeof(char*) * (count + 1));
 		*vals = g_realloc (*vals, sizeof(char*) * (count + 1));
-		
+
 		(*keys)[count-1] = g_strdup (key);
 		(*vals)[count-1] = g_strdup (val);
 
 		(*keys)[count] = NULL;
 		(*vals)[count] = NULL;
-		
+
 		next = va_arg (opts, gchar*);
 	}
 }
@@ -1365,7 +1273,7 @@ collect_save_options (va_list    opts,
 
 gboolean
 _gdk_pixbuf_save (GdkPixbuf    *pixbuf,
-		  const char   *filename,
+		  const char   *local_file,
 		  const char   *type,
 		  GError      **error,
 		  ...)
@@ -1375,18 +1283,19 @@ _gdk_pixbuf_save (GdkPixbuf    *pixbuf,
 	char     **values = NULL;
 	gboolean   result;
 
-	g_return_val_if_fail (pixbuf != NULL, TRUE);
-	g_return_val_if_fail (filename != NULL, TRUE);
-	g_return_val_if_fail (type != NULL, TRUE);
+	g_return_val_if_fail (pixbuf != NULL, FALSE);
+	g_return_val_if_fail (local_file != NULL, FALSE);
+	g_return_val_if_fail (type != NULL, FALSE);
+	g_return_val_if_fail (! uri_has_scheme (local_file), FALSE);
 
 	va_start (args, error);
 	collect_save_options (args, &keys, &values);
 	va_end (args);
 
-	result = _gdk_pixbuf_savev (pixbuf, filename, type,
+	result = _gdk_pixbuf_savev (pixbuf, local_file, type,
 				    keys, values,
 				    error);
-	
+
 	g_strfreev (keys);
 	g_strfreev (values);
 
@@ -1408,13 +1317,13 @@ scale_keepping_ratio (int *width,
 	int      new_width, new_height;
 	gboolean modified;
 
-	if ((*width < max_width) && (*height < max_height)) 
+	if ((*width < max_width) && (*height < max_height))
 		return FALSE;
 
 	factor = MIN (max_w / w, max_h / h);
 	new_width  = MAX ((int) floor (w * factor + 0.50), 1);
 	new_height = MAX ((int) floor (h * factor + 0.50), 1);
-	
+
 	modified = (new_width != *width) || (new_height != *height);
 
 	*width = new_width;
