@@ -190,7 +190,7 @@ orientation_is_portrait (GtkPageSetup *page_setup)
 
 
 typedef struct {
-	char        *filename;
+	FileData    *file;
 	char        *comment;
 	int          pixbuf_width, pixbuf_height;
 	GdkPixbuf   *thumbnail;
@@ -212,11 +212,11 @@ typedef struct {
 
 
 static ImageInfo *
-image_info_new (const char *filename)
+image_info_new (FileData *file)
 {
-	ImageInfo *image = g_new0(ImageInfo, 1);
+	ImageInfo *image = g_new0 (ImageInfo, 1);
 
-	image->filename = g_strdup (filename);
+	image->file = file_data_ref (file);
 	image->comment = NULL;
 	image->thumbnail = NULL;
 	image->thumbnail_active = NULL;
@@ -244,7 +244,7 @@ image_info_free (ImageInfo *image)
 {
 	g_return_if_fail (image != NULL);
 
-	g_free (image->filename);
+	file_data_unref (image->file);
 	g_free (image->comment);
 	if (image->thumbnail != NULL)
 		g_object_unref (image->thumbnail);
@@ -655,11 +655,11 @@ construct_comment (PrintCatalogInfo *pci,
 
 	if (pci->print_filenames) {
 		const gchar* end = NULL;
-		g_utf8_validate (image->filename, -1, &end);
-		if (end > image->filename) {
+		g_utf8_validate (image->file->path, -1, &end);
+		if (end > image->file->path) {
 			if (s->len > 0)
 				g_string_append (s, "\n");
-			g_string_append_len (s, image->filename, end - image->filename);
+			g_string_append_len (s, image->file->path, end - image->file->path);
 		}
 	}
 
@@ -792,9 +792,6 @@ pci_print_comment (PrintCatalogInfo *pci,
 }
 
 
-
-
-
 static int catalog_rows[5] = {1, 2, 2, 4, 4};
 static int catalog_cols[5] = {1, 1, 2, 2, 4};
 #define IMAGE_SPACE 20
@@ -840,13 +837,13 @@ print__gdk_pixbuf_rotate (GdkPixbuf *pixbuf,
 		g_object_ref (rotated);
 		break;
 	case 90:
-		rotated = _gdk_pixbuf_copy_rotate_90 (pixbuf, FALSE);
+		rotated = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_CLOCKWISE);
 		break;
 	case 180:
-		rotated = _gdk_pixbuf_copy_mirror (pixbuf, TRUE, TRUE);
+		rotated = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_UPSIDEDOWN);
 		break;
 	case 270:
-		rotated = _gdk_pixbuf_copy_rotate_90 (pixbuf, TRUE);
+		rotated = gdk_pixbuf_rotate_simple (pixbuf, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
 		break;
 	}
 
@@ -1429,7 +1426,7 @@ catalog_update_custom_page_size (PrintCatalogDialogData *data)
 
 	size = gtk_paper_size_new_custom("customX", "customY", width, height, unit);
 	gtk_page_setup_set_paper_size (data->pci->page_setup, size);
-	g_free(size);
+	gtk_paper_size_free (size);
 
 	catalog_update_page (data);
 }
@@ -1677,7 +1674,7 @@ catalog_set_standard_page_size (PrintCatalogDialogData *data,
 
 	size = gtk_paper_size_new_custom (page_size, "", width, height, unit);
 	gtk_page_setup_set_paper_size (data->pci->page_setup, size);
-	g_free (size);
+	gtk_paper_size_free (size);
 
 	catalog_update_page_size_from_config (data);
 	catalog_update_image_size_from_config (data);
@@ -1717,6 +1714,7 @@ catalog_update_margins (PrintCatalogDialogData *data)
 	catalog_update_page (data);
 }
 
+
 static void
 done_print (GtkPrintOperation       *operation,
 	    GtkPrintOperationResult  result,
@@ -1725,6 +1723,7 @@ done_print (GtkPrintOperation       *operation,
 	print_catalog_info_unref (pci);
 }
 
+
 static void
 begin_print (GtkPrintOperation *operation,
 	     GtkPrintContext   *context,
@@ -1736,6 +1735,7 @@ begin_print (GtkPrintOperation *operation,
 
 }
 
+
 static gboolean
 preview (GtkPrintOperation        *operation,
 	 GtkPrintOperationPreview *preview,
@@ -1747,6 +1747,7 @@ preview (GtkPrintOperation        *operation,
 	return FALSE;
 }
 
+
 static void
 draw_page (GtkPrintOperation *operation,
 	   GtkPrintContext   *context,
@@ -1774,10 +1775,9 @@ draw_page (GtkPrintOperation *operation,
 	}
 
 	for (i = page_nr * pci->images_per_page; i < pci->n_images && i < (page_nr + 1) * pci->images_per_page; i++) {
-
 		ImageInfo *image = pci->image_info[i];
 		GdkPixbuf *image_pixbuf, *pixbuf = NULL;
-		double scale_factor;
+		double     scale_factor;
 
 		if (pci->print_comments || pci->print_filenames) {
 			cairo_save (cr);
@@ -1785,8 +1785,7 @@ draw_page (GtkPrintOperation *operation,
 			cairo_restore (cr);
 		}
 
-		image_pixbuf = gth_pixbuf_new_from_uri (image->filename, NULL, 0, 0, NULL);
-
+		image_pixbuf = gth_pixbuf_new_from_file (image->file, NULL, 0, 0, NULL);
 		pixbuf = print__gdk_pixbuf_rotate (image_pixbuf, image->rotate);
 		g_object_unref (image_pixbuf);
 
@@ -1886,6 +1885,7 @@ draw_page (GtkPrintOperation *operation,
 	}
 }
 
+
 static const char *
 catalog_get_current_paper_size (PrintCatalogDialogData *data)
 {
@@ -2143,8 +2143,6 @@ print_catalog_cb (GtkWidget              *widget,
 	/* pci is used to print, so we must add a reference. */
 
 	print_catalog_info_ref (pci);
-	gtk_widget_hide (data->dialog);
-
 
 	/* GTK Printing */
 	if (!pci->print_operation)
@@ -2159,13 +2157,15 @@ print_catalog_cb (GtkWidget              *widget,
 
 	pci->is_preview = FALSE;
 
+	/* Hide the setup dialog while the gtk-print dialog is shown */
+	gtk_widget_hide (data->dialog);
+
 	result = gtk_print_operation_run (pci->print_operation, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW (data->parent), &error);
 
 	g_object_unref (pci->print_operation);
 	pci->print_operation = NULL;
 
-	if (result == GTK_PRINT_OPERATION_RESULT_ERROR)
-	{
+	if (result == GTK_PRINT_OPERATION_RESULT_ERROR) {
 		GtkWidget* error_dialog = gtk_message_dialog_new (GTK_WINDOW (data->parent),
 						       GTK_DIALOG_DESTROY_WITH_PARENT,
 						       GTK_MESSAGE_ERROR,
@@ -2177,12 +2177,11 @@ print_catalog_cb (GtkWidget              *widget,
 		gtk_widget_show (error_dialog);
 		g_error_free (error);
 	}
-	else if (result == GTK_PRINT_OPERATION_RESULT_APPLY)
-	{
+	else if (result == GTK_PRINT_OPERATION_RESULT_APPLY) {
 		/* printing completed successfully  */
 	}
-	else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL)
-	{
+	else if (result == GTK_PRINT_OPERATION_RESULT_CANCEL) {
+		/* return to the setup dialog */
 		gtk_widget_show (data->dialog);
 		return;
 	}
@@ -2190,10 +2189,12 @@ print_catalog_cb (GtkWidget              *widget,
 		/* print in progress */
 	}
 
-	if (pci->is_preview)
-		gtk_widget_show (data->dialog);
-	else
+	if (! pci->is_preview)
+		/* Done printing */
 		gtk_widget_destroy (data->dialog);
+	else
+		/* Done preview, ready to really print */
+		gtk_widget_show (data->dialog);
 }
 
 
@@ -2230,8 +2231,8 @@ prev_page_cb (GtkWidget              *widget,
 static void
 load_current_image (PrintCatalogDialogData *data)
 {
-	const char *filename;
-	char       *msg;
+	FileData *file;
+	char     *msg;
 
 	if (data->current_image >= data->pci->n_images) {
 		progress_dialog_hide (data->pd);
@@ -2243,14 +2244,13 @@ load_current_image (PrintCatalogDialogData *data)
 
 	progress_dialog_set_progress (data->pd, (double) data->current_image / data->pci->n_images);
 
-	filename = data->pci->image_info[data->current_image]->filename;
+	file = data->pci->image_info[data->current_image]->file;
 
-	msg = g_strdup_printf (_("Loading image: %s"), file_name_from_path (filename));
+	msg = g_strdup_printf (_("Loading image: %s"), basename_for_display (file->path));
 	progress_dialog_set_info (data->pd, msg);
 	g_free (msg);
 
-	image_loader_set_path (data->loader, filename, NULL);
-
+	image_loader_set_file (data->loader, file);
 	image_loader_start (data->loader);
 }
 
@@ -2333,7 +2333,7 @@ image_loader_done_cb (ImageLoader            *il,
 		}
 	}
 
-	cdata = comments_load_comment (image->filename, TRUE);
+	cdata = comments_load_comment (image->file->path, TRUE);
 	if (cdata != NULL) {
 		image->comment = comments_get_comment_as_string (cdata, "\n", " - ");
 		comment_data_free (cdata);
@@ -2421,8 +2421,8 @@ print_catalog_dlg_full (GtkWindow *parent,
 	pci->n_images = g_list_length (file_list);
 	pci->image_info = g_new (ImageInfo*, pci->n_images);
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = scan->data;
-		pci->image_info[n++] = image_info_new (filename);
+		FileData *file = scan->data;
+		pci->image_info[n++] = image_info_new (file);
 	}
 
 	data = g_new0 (PrintCatalogDialogData, 1);
@@ -2438,7 +2438,7 @@ print_catalog_dlg_full (GtkWindow *parent,
 	data->done_func = done_func;
 	data->done_data = done_data;
 
-	data->loader = IMAGE_LOADER (image_loader_new (NULL, FALSE));
+	data->loader = IMAGE_LOADER (image_loader_new (FALSE));
 	g_signal_connect (G_OBJECT (data->loader),
 			  "image_done",
 			  G_CALLBACK (image_loader_done_cb),
@@ -2513,7 +2513,6 @@ print_catalog_dlg_full (GtkWindow *parent,
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->margin_bottom_spinbutton), eel_gconf_get_float (PREF_PRINT_MARGIN_BOTTOM, DEF_MARGIN_BOTTOM));
 	gtk_option_menu_set_history (GTK_OPTION_MENU (data->unit_optionmenu), pref_get_print_unit ());
 
-
 	gtk_option_menu_set_history (GTK_OPTION_MENU (data->resolution_optionmenu), pref_get_image_resolution());
 	gtk_option_menu_set_history (GTK_OPTION_MENU (data->img_unit_optionmenu), pref_get_image_unit ());
 	switch (pref_get_image_sizing()) {
