@@ -100,6 +100,30 @@ static char *get_header_text                (CatalogPngExporter *ce,
 					     int                 page_n);
 
 
+#define HTML_PAGE_BEGIN "\
+<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\
+<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\
+  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n\
+<html xmlns=\"http://www.w3.org/1999/xhtml\">\n\
+<head>\n\
+<title></title>\n\
+<style type=\"text/css\">\n\
+html { margin: 0px; border: 0px; padding: 0px; }\n\
+body { margin: 0px; }\n\
+img  { border: 0px; }\n\
+</style>\n\
+</head>\n\
+<body>\n\
+<div>\n\
+"
+
+#define HTML_PAGE_END "\
+</div>\n\
+</body>\n\
+</html>\n\
+"
+
+
 #define COL_SPACING      15
 #define ROW_SPACING      15
 #define THUMB_FRAME_BORDER     8
@@ -109,10 +133,8 @@ static char *get_header_text                (CatalogPngExporter *ce,
 #define DEFAULT_FONT "Sans 12"
 
 typedef struct {
+	FileData         *file;
 	char             *comment;
-	char             *filename;
-	GnomeVFSFileSize  file_size;
-	time_t            file_time;
 	time_t		  exif_time;
 	GdkPixbuf        *thumb;
 	int               image_width;
@@ -125,21 +147,20 @@ typedef struct {
 
 
 static ImageData *
-image_data_new (gchar *filename)
+image_data_new (FileData *file)
 {
 	ImageData   *idata;
 	CommentData *cdata;
 	int          i;
 
-	idata = g_new (ImageData, 1);
+	idata = g_new0 (ImageData, 1);
 
-	cdata = comments_load_comment (filename, TRUE);
-	idata->comment = comments_get_comment_as_string (cdata, "\n", "\n");
-	if (cdata != NULL)
+	idata->file = file_data_ref (file);
+	cdata = comments_load_comment (file->path, TRUE);
+	if (cdata != NULL) {
+		idata->comment = comments_get_comment_as_string (cdata, "\n", "\n");
 		comment_data_free (cdata);
-
-	idata->filename = g_strdup (filename);
-	idata->file_size = 0;
+	}
 	idata->thumb = NULL;
 	idata->image_width = 0;
 	idata->image_height = 0;
@@ -158,7 +179,7 @@ image_data_free (ImageData *idata)
 	int i;
 
 	g_free (idata->comment);
-	g_free (idata->filename);
+	file_data_unref (idata->file);
 
 	if (idata->thumb)
 		g_object_unref (G_OBJECT (idata->thumb));
@@ -191,9 +212,9 @@ catalog_png_exporter_finalize (GObject *object)
 
         ce = CATALOG_PNG_EXPORTER (object);
 
-	if (ce->directory != NULL) {
-		g_free (ce->directory);
-		ce->directory = NULL;
+	if (ce->location != NULL) {
+		g_free (ce->location);
+		ce->location = NULL;
 	}
 
 	if (ce->info != NULL) {
@@ -222,9 +243,9 @@ catalog_png_exporter_finalize (GObject *object)
 		ce->file_list = NULL;
 	}
 
-	if (ce->created_list != NULL) {
-		path_list_free (ce->created_list);
-		ce->created_list = NULL;
+	if (ce->created_files != NULL) {
+		path_list_free (ce->created_files);
+		ce->created_files = NULL;
 	}
 
 	if (ce->pages_height != NULL) {
@@ -272,6 +293,11 @@ catalog_png_exporter_finalize (GObject *object)
 		ce->iloader = NULL;
 	}
 
+	if (ce->imap_uri != NULL) {
+		g_free (ce->imap_uri);
+		ce->imap_uri = NULL;
+	}
+
 	/* Chain up */
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -325,7 +351,7 @@ catalog_png_exporter_init (CatalogPngExporter *ce)
 	PangoFontDescription *font_desc;
 
 	ce->file_list = NULL;
-	ce->created_list = NULL;
+	ce->created_files = NULL;
 
 	ce->page_width = 0;
 	ce->page_height = 0;
@@ -375,7 +401,7 @@ catalog_png_exporter_init (CatalogPngExporter *ce)
 
 	ce->frame_style = GTH_FRAME_STYLE_SIMPLE;
 
-	ce->directory = NULL;
+	ce->location = NULL;
 	ce->name_template = NULL;
 	ce->templatev = NULL;
 	ce->file_type = g_strdup ("png");
@@ -385,6 +411,7 @@ catalog_png_exporter_init (CatalogPngExporter *ce)
 
 	ce->write_image_map = FALSE;
 	ce->imap_handle = NULL;
+	ce->imap_uri = NULL;
 
 	ce->info = NULL;
 
@@ -449,7 +476,7 @@ catalog_png_exporter_new (GList *file_list)
 	for (scan = file_list; scan; scan = scan->next) {
 		ImageData *idata;
 
-		idata = image_data_new ((gchar*) scan->data);
+		idata = image_data_new ((FileData*) scan->data);
 		ce->file_list = g_list_prepend (ce->file_list, idata);
 	}
 	ce->file_list = g_list_reverse (ce->file_list);
@@ -459,14 +486,13 @@ catalog_png_exporter_new (GList *file_list)
 
 
 void
-catalog_png_exporter_set_directory (CatalogPngExporter *ce,
-				    char               *directory)
+catalog_png_exporter_set_location (CatalogPngExporter *ce,
+				   const char         *location)
 {
 	g_return_if_fail (IS_CATALOG_PNG_EXPORTER (ce));
 
-	if (ce->directory)
-		g_free (ce->directory);
-	ce->directory = g_strdup (directory);
+	g_free (ce->location);
+	ce->location = g_strdup (location);
 }
 
 
@@ -795,23 +821,25 @@ set_item_caption (CatalogPngExporter *ce,
 
 	if ((ce->caption_fields & GTH_CAPTION_FILE_PATH)
 	    && (ce->caption_fields & GTH_CAPTION_FILE_NAME)) {
-		char *utf8_name = gnome_vfs_unescape_string_for_display (idata->filename);
+		char *utf8_name = gnome_vfs_unescape_string_for_display (idata->file->path);
 		idata->caption_row[row++] = utf8_name;
-	} else {
+	} 
+	else {
 		if (ce->caption_fields & GTH_CAPTION_FILE_PATH) {
-			char *path = remove_level_from_path (idata->filename);
+			char *path = remove_level_from_path (idata->file->path);
 			char *utf8_name = gnome_vfs_unescape_string_for_display (path);
 			idata->caption_row[row++] = utf8_name;
 			g_free (path);
-		} else if (ce->caption_fields & GTH_CAPTION_FILE_NAME) {
-			const char *name = file_name_from_path (idata->filename);
+		} 
+		else if (ce->caption_fields & GTH_CAPTION_FILE_NAME) {
+			const char *name = file_name_from_path (idata->file->path);
 			char *utf8_name = gnome_vfs_unescape_string_for_display (name);
 			idata->caption_row[row++] = utf8_name;
 		}
 	}
 
 	if (ce->caption_fields & GTH_CAPTION_FILE_SIZE)
-		idata->caption_row[row++] = gnome_vfs_format_file_size_for_display (idata->file_size);
+		idata->caption_row[row++] = gnome_vfs_format_file_size_for_display (idata->file->size);
 
 	if (ce->caption_fields & GTH_CAPTION_IMAGE_DIM)
 		idata->caption_row[row++] = g_strdup_printf (
@@ -839,6 +867,60 @@ get_page_height (CatalogPngExporter *ce,
 }
 
 
+static void copy_current_file_to_destination (CatalogPngExporter *ce);
+
+
+static void
+copy_current_file_to_destination_done (const char     *uri, 
+				       GnomeVFSResult  result, 
+				       gpointer        data)
+{
+	CatalogPngExporter *ce = data;
+	
+	ce->current_file = ce->current_file->next;
+	copy_current_file_to_destination (ce);
+}
+
+
+static void
+copy_current_file_to_destination (CatalogPngExporter *ce)
+{
+	FileData *file;
+	
+	if (ce->current_file == NULL) {
+		end_export (ce);
+		return;
+	}
+	
+	g_signal_emit (G_OBJECT (ce),
+		       catalog_png_exporter_signals[PNG_EXPORTER_PROGRESS],
+		       0,
+		       ((float) ++ce->n_files_done) / (ce->n_files + 1));	
+
+	file = file_data_new ((char*) ce->current_file->data, NULL);
+	update_file_from_cache (file, copy_current_file_to_destination_done, ce);
+	
+	file_data_unref (file);
+}
+
+
+static void
+copy_created_files_to_destination (CatalogPngExporter *ce)
+{
+	g_free (ce->info);
+	ce->info = g_strdup (_("Saving images"));
+	g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[PNG_EXPORTER_INFO],
+		       0,
+		       ce->info);
+		       	
+	ce->n_files_done = 0;
+	ce->n_files = g_list_length (ce->created_files);
+	ce->current_file = ce->created_files;
+	
+	copy_current_file_to_destination (ce);
+}
+
+
 static void
 export (CatalogPngExporter *ce)
 {
@@ -876,7 +958,6 @@ export (CatalogPngExporter *ce)
 				g_list_free (ce->file_list);
 				ce->file_list = NULL;
 			}
-			g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[PNG_EXPORTER_DONE], 0);
 			goto label_end;
 		}
 
@@ -961,13 +1042,11 @@ export (CatalogPngExporter *ce)
 		for (scan_row = row_first_item; scan_row != row_last_item; scan_row = scan_row->next) {
 			GdkPixbuf     *pixbuf;
 			char         **row;
-			char          *filename;
 			int            y1, h, i;
 			GdkRectangle   frame_rect, image_rect;
 			ImageData     *row_idata = IMAGE_DATA (scan_row->data);
 
 			row = row_idata->caption_row;
-			filename = row_idata->filename;
 
 			frame_rect.x = x;
 			frame_rect.y = y;
@@ -987,7 +1066,7 @@ export (CatalogPngExporter *ce)
 				image_rect.height = theight;
 
 				paint_frame (ce, &frame_rect, &image_rect,
-					     filename);
+					     row_idata->file->path);
 				paint_image (ce, &image_rect, pixbuf);
 			}
 
@@ -1019,8 +1098,7 @@ export (CatalogPngExporter *ce)
 	} while (TRUE);
 
  label_end:
-	end_export (ce);
-	g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[PNG_EXPORTER_DONE], 0);
+	copy_created_files_to_destination (ce);
 }
 
 
@@ -1131,7 +1209,7 @@ compute_pages_size (CatalogPngExporter *ce)
 
 	cols = ce->page_cols;
 	rows = ce->page_rows;
-	pages = ce->n_images / (cols * rows) + 2;
+	pages = ce->n_files / (cols * rows) + 2;
 	ce->pages_height = g_new (int, pages);
 	ce->pages_n = 0;
 
@@ -1200,7 +1278,7 @@ comp_func_name (gconstpointer a, gconstpointer b)
 	data_a = IMAGE_DATA (a);
 	data_b = IMAGE_DATA (b);
 
-	return gth_sort_by_filename_but_ignore_path (data_a->filename,data_b->filename);
+	return gth_sort_by_filename_but_ignore_path (data_a->file->path, data_b->file->path);
 }
 
 
@@ -1212,7 +1290,7 @@ comp_func_path (gconstpointer a, gconstpointer b)
 	data_a = IMAGE_DATA (a);
 	data_b = IMAGE_DATA (b);
 
-	return gth_sort_by_full_path (data_a->filename, data_b->filename);
+	return gth_sort_by_full_path (data_a->file->path, data_b->file->path);
 }
 
 
@@ -1225,7 +1303,7 @@ comp_func_comment (gconstpointer a, gconstpointer b)
 	data_b = IMAGE_DATA (b);
 
 	return gth_sort_by_comment_then_name (data_a->comment, data_b->comment,
-					data_a->filename, data_b->filename);
+					data_a->file->path, data_b->file->path);
 }
 
 
@@ -1237,8 +1315,8 @@ comp_func_time (gconstpointer a, gconstpointer b)
 	data_a = IMAGE_DATA (a);
 	data_b = IMAGE_DATA (b);
 
-	return gth_sort_by_filetime_then_name (data_a->file_time, data_b->file_time,
-						data_a->filename, data_b->filename);
+	return gth_sort_by_filetime_then_name (data_a->file->mtime, data_b->file->mtime,
+						data_a->file->path, data_b->file->path);
 }
 
 
@@ -1251,7 +1329,7 @@ comp_func_exif_date (gconstpointer a, gconstpointer b)
 	data_b = IMAGE_DATA (b);
 
 	return gth_sort_by_filetime_then_name (data_a->exif_time, data_b->exif_time,
-						data_a->filename, data_b->filename);
+						data_a->file->path, data_b->file->path);
 }
 
 
@@ -1263,7 +1341,7 @@ comp_func_size (gconstpointer a, gconstpointer b)
 	data_a = IMAGE_DATA (a);
 	data_b = IMAGE_DATA (b);
 
-	return gth_sort_by_size_then_name (data_a->file_size, data_b->file_size, data_a->filename, data_b->filename);
+	return gth_sort_by_size_then_name (data_a->file->size, data_b->file->size, data_a->file->path, data_b->file->path);
 }
 
 
@@ -1306,8 +1384,7 @@ get_sortfunc (CatalogPngExporter *ce)
 static void
 load_next_file (CatalogPngExporter *ce)
 {
-	char *filename;
-	char *utf8_name;
+	ImageData *idata;
 
 	if (ce->interrupted) {
 		if (ce->file_list != NULL) {
@@ -1322,7 +1399,7 @@ load_next_file (CatalogPngExporter *ce)
 	g_signal_emit (G_OBJECT (ce),
 		       catalog_png_exporter_signals[PNG_EXPORTER_PROGRESS],
 		       0,
-		       ((float) ++ce->n_images_done) / ce->n_images);
+		       ((float) ++ce->n_files_done) / (ce->n_files + 1));
 
 	ce->file_to_load = ce->file_to_load->next;
 	if (ce->file_to_load == NULL) {
@@ -1342,21 +1419,18 @@ load_next_file (CatalogPngExporter *ce)
 		return;
 	}
 
-	filename = IMAGE_DATA (ce->file_to_load->data)->filename;
-	image_loader_set_path (ce->iloader, filename, NULL);
-	image_loader_start (ce->iloader);
+	/* load the file */
 
-	/* info */
+	idata = IMAGE_DATA (ce->file_to_load->data);
 
 	g_free (ce->info);
-
-	utf8_name = gnome_vfs_unescape_string_for_display (file_name_from_path (filename));
-	ce->info = g_strdup_printf (_("Loading image: %s"), utf8_name);
-	g_free (utf8_name);
-
+	ce->info = g_strdup_printf (_("Loading image: %s"), idata->file->display_name);
 	g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[PNG_EXPORTER_INFO],
 		       0,
 		       ce->info);
+
+	image_loader_set_file (ce->iloader, idata->file);
+	image_loader_start (ce->iloader);
 }
 
 
@@ -1375,14 +1449,9 @@ image_loader_done (ImageLoader *iloader,
 	idata->image_width = gdk_pixbuf_get_width (pixbuf);
 	idata->image_height = gdk_pixbuf_get_height (pixbuf);
 
-	/* file size. */
-	idata->file_size = get_file_size (idata->filename);
-
-	/* file time. */
-	idata->file_time = get_file_mtime (idata->filename);
-
 	/* time in exif tag, if present */
-	idata->exif_time = get_exif_time (idata->filename);
+
+	idata->exif_time = get_metadata_time (NULL, idata->file->path);
 
 	/* thumbnail. */
 	idata->thumb = pixbuf = image_loader_get_pixbuf (iloader);
@@ -1435,12 +1504,12 @@ catalog_png_exporter_export (CatalogPngExporter *ce)
 	if (ce->iloader != NULL)
 		g_object_unref (ce->iloader);
 
-	if (ce->created_list != NULL) {
-		path_list_free (ce->created_list);
-		ce->created_list = NULL;
+	if (ce->created_files != NULL) {
+		path_list_free (ce->created_files);
+		ce->created_files = NULL;
 	}
 
-	ce->iloader = IMAGE_LOADER (image_loader_new (NULL, FALSE));
+	ce->iloader = IMAGE_LOADER (image_loader_new (FALSE));
 	g_signal_connect (G_OBJECT (ce->iloader),
 			  "image_done",
 			  G_CALLBACK (image_loader_done),
@@ -1450,13 +1519,12 @@ catalog_png_exporter_export (CatalogPngExporter *ce)
 			  G_CALLBACK (image_loader_error),
 			  ce);
 
-	ce->n_images = g_list_length (ce->file_list);
-	ce->n_images_done = 0;
+	ce->n_files = g_list_length (ce->file_list);
+	ce->n_files_done = 0;
 
 	ce->file_to_load = ce->file_list;
-	image_loader_set_path (ce->iloader,
-			       IMAGE_DATA (ce->file_to_load->data)->filename,
-			       NULL);
+	image_loader_set_file (ce->iloader,
+			       IMAGE_DATA (ce->file_to_load->data)->file);
 	image_loader_start (ce->iloader);
 }
 
@@ -1472,7 +1540,6 @@ catalog_png_exporter_interrupt (CatalogPngExporter *ce)
 }
 
 
-
 /* ----- */
 
 
@@ -1495,17 +1562,19 @@ begin_export (CatalogPngExporter *ce)
 
 static void
 end_export (CatalogPngExporter *ce)
-{
-	if (ce->created_list != NULL) {
+{	
+	if (ce->created_files != NULL) {
 		all_windows_remove_monitor ();
-		all_windows_notify_files_created (ce->created_list);
+		all_windows_notify_files_created (ce->created_files);
 		all_windows_add_monitor ();
-		path_list_free (ce->created_list);
-		ce->created_list = NULL;
+		path_list_free (ce->created_files);
+		ce->created_files = NULL;
 	}
 
 	g_object_unref (G_OBJECT (ce->gc));
 	g_object_unref (G_OBJECT (ce->pixmap));
+
+	g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[PNG_EXPORTER_DONE], 0);
 }
 
 
@@ -1559,12 +1628,13 @@ static void
 begin_page (CatalogPngExporter *ce,
 	    int                 page_n)
 {
-	GnomeVFSURI      *uri;
+	GnomeVFSURI      *vfs_uri;
 	GnomeVFSResult    result;
 	GnomeVFSFileSize  temp;
 	int               width, height;
-	char             *path;
+	char             *local_file;
 	char             *filename;
+	char             *name;
 	char             *line;
 	char             *utf8_name;
 
@@ -1599,37 +1669,39 @@ begin_page (CatalogPngExporter *ce,
 	if (! ce->write_image_map)
 		return;
 
-	filename = _g_get_name_from_template (ce->templatev,
-					      ce->start_at + page_n - 1);
-	path = g_strconcat (ce->directory, "/", filename, ".html", NULL);
-
-	uri = new_uri_from_path (path);
-	g_free (path);
-
-	if (uri == NULL) {
-		g_warning ("URI not valid: %s", path);
-		return;
-	}
-
+	g_free (ce->imap_uri);
+	ce->imap_uri = NULL;
 	ce->imap_handle = NULL;
-	result = gnome_vfs_create_uri (&ce->imap_handle, uri,
-				       GNOME_VFS_OPEN_WRITE, FALSE, 0664);
-	gnome_vfs_uri_unref (uri);
+	
+	name = _g_get_name_from_template (ce->templatev, ce->start_at + page_n - 1);
+	ce->imap_uri = g_strconcat (ce->location, "/", name, ".html", NULL);
 
-	if (result != GNOME_VFS_OK) {
-		g_warning ("Cannot create file %s", path);
+	local_file = get_cache_uri_from_uri (ce->imap_uri);
+	vfs_uri = new_uri_from_path (local_file);
+	g_free (local_file);
+	
+	if (vfs_uri == NULL) {
+		g_warning ("URI not valid: %s", local_file);
 		return;
 	}
 
-	path = g_strconcat (filename, ".", ce->file_type, NULL);
-	g_free (filename);
+	result = gnome_vfs_create_uri (&ce->imap_handle, vfs_uri,
+				       GNOME_VFS_OPEN_WRITE, FALSE, 0664);
+	gnome_vfs_uri_unref (vfs_uri);
+	if (result != GNOME_VFS_OK) {
+		g_warning ("Cannot create file %s", ce->imap_uri);
+		return;
+	}
 
-	line = g_strdup_printf ("<IMG SRC=\"%s\" WIDTH=%d HEIGHT=%d BORDER=0 USEMAP=\"#map\">\n\n", path, width, height);
+	gnome_vfs_write (ce->imap_handle, HTML_PAGE_BEGIN, strlen (HTML_PAGE_BEGIN), &temp);
+
+	filename = g_strconcat (name, ".", ce->file_type, NULL);
+	line = g_strdup_printf ("<img src=\"%s\" width=\"%d\" height=\"%d\" usemap=\"#map\" alt=\"%s\" />\n", filename, width, height, filename);
 	gnome_vfs_write (ce->imap_handle, line, strlen (line), &temp);
 	g_free (line);
-	g_free (path);
+	g_free (filename);
 
-	line = g_strdup_printf ("<MAP NAME=\"map\">\n");
+	line = g_strdup_printf ("<map name=\"map\" id=\"map\">\n");
 	gnome_vfs_write (ce->imap_handle, line, strlen (line), &temp);
 	g_free (line);
 }
@@ -1640,8 +1712,9 @@ end_page (CatalogPngExporter *ce,
 	  int                 page_n)
 {
 	GdkPixbuf        *pixbuf;
-	char             *path;
-	char             *filename;
+	char             *name;
+	char             *uri;
+	char             *local_file;
 	int               width, height;
 	GnomeVFSFileSize  temp;
 	char             *line;
@@ -1657,30 +1730,34 @@ end_page (CatalogPngExporter *ce,
 					       width,
 					       height);
 
-	filename = _g_get_name_from_template (ce->templatev, ce->start_at + page_n - 1);
-	path = g_strconcat (ce->directory, "/", filename, ".", ce->file_type, NULL);
-	g_free (filename);
-
+	name = _g_get_name_from_template (ce->templatev, ce->start_at + page_n - 1);
+	uri = g_strconcat (ce->location, "/", name, ".", ce->file_type, NULL);
+	local_file = get_cache_filename_from_uri (uri);
+	
 	if (strcmp (ce->file_type, "jpeg") == 0)
-		_gdk_pixbuf_save (pixbuf, path, "jpeg", NULL, "quality", "85", NULL);
+		_gdk_pixbuf_save (pixbuf, local_file, "jpeg", NULL, "quality", "85", NULL);
 	else
-		_gdk_pixbuf_save (pixbuf, path, ce->file_type, NULL, NULL);
+		_gdk_pixbuf_save (pixbuf, local_file, ce->file_type, NULL, NULL);
 
-	ce->created_list = g_list_prepend (ce->created_list, g_strdup (path));
+	ce->created_files = g_list_prepend (ce->created_files, uri);
 
+	g_free (local_file);
+	g_free (name);
 	g_object_unref (pixbuf);
-	g_free (path);
-
+	
 	/* image map file. */
 
 	if (! ce->write_image_map || (ce->imap_handle == NULL))
 		return;
 
-	line = g_strdup_printf ("</MAP>\n");
+	line = g_strdup_printf ("</map>\n");
 	gnome_vfs_write (ce->imap_handle, line, strlen (line), &temp);
-	g_free (line);
-
+	gnome_vfs_write (ce->imap_handle, HTML_PAGE_END, strlen (HTML_PAGE_END), &temp);
 	gnome_vfs_close (ce->imap_handle);
+	
+	ce->created_files = g_list_prepend (ce->created_files, g_strdup (ce->imap_uri));
+	
+	g_free (line);
 }
 
 
@@ -1691,6 +1768,7 @@ paint_frame (CatalogPngExporter *ce,
 	     gchar              *filename)
 {
 	GnomeVFSFileSize  temp;
+	char		 *unescaped_path, *attr_path;
 	char             *line;
 	char             *rel_path;
 	char             *dest_dir;
@@ -1764,17 +1842,23 @@ paint_frame (CatalogPngExporter *ce,
 	if (! ce->write_image_map || (ce->imap_handle == NULL))
 		return;
 
-	dest_dir = remove_special_dirs_from_path (ce->directory);
-	rel_path = get_path_relative_to_dir (filename, dest_dir);
+	dest_dir = remove_special_dirs_from_path (ce->location);
+	rel_path = get_path_relative_to_uri (filename, dest_dir);
 	g_free (dest_dir);
 
-	line = g_strdup_printf ("<AREA SHAPE=\"RECT\" COORDS=\"%d,%d,%d,%d\" HREF=\"%s\">\n",
+	unescaped_path = gnome_vfs_unescape_string (rel_path, NULL);
+	attr_path = _g_escape_text_for_html (unescaped_path, -1);
+	g_free (unescaped_path);
+
+	line = g_strdup_printf ("<area shape=\"rect\" coords=\"%d,%d,%d,%d\" href=\"%s\" alt=\"%s\" />\n",
 				frame_rect->x,
 				frame_rect->y,
 				frame_rect->x + frame_rect->width,
 				frame_rect->y + frame_rect->height,
-				rel_path);
+				rel_path,
+				attr_path);
 	g_free (rel_path);
+	g_free (attr_path);
 	gnome_vfs_write (ce->imap_handle, line, strlen (line), &temp);
 	g_free (line);
 }
