@@ -806,7 +806,8 @@ dlg_duplicates (GthBrowser *browser)
 
 	/* Set widgets data. */
 
-	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->fd_start_from_filechooserbutton), gth_browser_get_current_directory (data->browser));
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (data->fd_start_from_filechooserbutton),
+						 gth_browser_get_current_directory (data->browser));
 
 	/* * Images model */
 
@@ -911,7 +912,6 @@ dlg_duplicates (GthBrowser *browser)
 }
 
 
-
 /* ---- */
 
 
@@ -1037,7 +1037,7 @@ start_loading_image (DialogData *data)
 	idata = data->loader_queue->data;
 
 	data->loading_image = TRUE;
-	thumb_loader_set_path (data->loader, idata->path, NULL);
+	thumb_loader_set_path (data->loader, idata->path);
 	thumb_loader_start (data->loader);
 }
 
@@ -1047,7 +1047,7 @@ queue_image_to_load (DialogData *data,
 		     ImageData  *idata)
 {
 	if (data->loader == NULL) {
-		data->loader = THUMB_LOADER (thumb_loader_new (NULL, MINI_IMAGE_SIZE, MINI_IMAGE_SIZE));
+		data->loader = THUMB_LOADER (thumb_loader_new (MINI_IMAGE_SIZE, MINI_IMAGE_SIZE));
 		thumb_loader_use_cache (data->loader, TRUE);
 		g_signal_connect (G_OBJECT (data->loader),
 				  "thumb_done",
@@ -1491,7 +1491,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 		case GNOME_VFS_FILE_TYPE_REGULAR:
 			full_uri = gnome_vfs_uri_append_file_name (data->uri, info->name);
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
-			if (file_is_image_or_video (str_uri, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, FALSE)))
+			if (file_is_image_video_or_audio (str_uri, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, FALSE)))
 				files = g_list_prepend (files, str_uri);
 			else
 				g_free (str_uri);
@@ -1554,7 +1554,7 @@ search_dir_async (DialogData *data, const char *path)
 		& (data->handle),
 		data->uri,
 		GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
-		128 /* items_per_notification FIXME */,
+		128 /* items_per_notification FIXME: find a good value */,
 		GNOME_VFS_PRIORITY_MIN,
 		directory_load_cb,
 		data);
