@@ -494,7 +494,7 @@ update_notebook_page (DialogData *data,
 	case IPROP_PAGE_EXIF:
 		gth_exif_data_viewer_update (GTH_EXIF_DATA_VIEWER (data->i_exif_data_view),
 					     data->browser_viewer,
-					     gth_window_get_image_filename (GTH_WINDOW (data->browser)),
+					     gth_window_get_image_data (GTH_WINDOW (data->browser)),
 					     NULL);
 		break;
 
@@ -528,7 +528,6 @@ i_notebook_switch_page_cb (GtkWidget       *widget,
 }
 
 
-
 GtkWidget *
 dlg_image_prop_new (GthBrowser *browser)
 {
