@@ -29,6 +29,7 @@
 #include <libgnomecanvas/libgnomecanvas.h>
 
 #include "glib-utils.h"
+#include "gtk-utils.h"
 #include "gth-utils.h"
 #include "gth-window.h"
 #include "gconf-utils.h"
@@ -500,13 +501,13 @@ history_changed_cb (GthImageHistory *history,
 void
 dlg_redeye_removal (GthWindow *window)
 {
-	DialogData *data;
-	GtkWidget  *ok_button;
-	GtkWidget  *save_button;
-	GtkWidget  *cancel_button;
-	GtkWidget  *help_button;
-	GtkWidget  *zoom_in_button, *zoom_out_button, *zoom_100_button, *zoom_fit_button;
-	GdkScreen  *screen;
+	DialogData   *data;
+	GtkWidget    *ok_button;
+	GtkWidget    *save_button;
+	GtkWidget    *cancel_button;
+	GtkWidget    *help_button;
+	GtkWidget    *zoom_in_button, *zoom_out_button, *zoom_100_button, *zoom_fit_button;
+	int	      width, height;
 
 	data = g_new0 (DialogData, 1);
 	data->window = window;
@@ -624,10 +625,11 @@ dlg_redeye_removal (GthWindow *window)
 
 	gtk_widget_realize (data->dialog);
 
-	screen = gdk_screen_get_default();
+	get_screen_size (GTK_WINDOW (window), &width, &height);
+
 	gtk_window_set_default_size (GTK_WINDOW (data->dialog),
-				     gdk_screen_get_width (screen) * 7 / 10,
-				     gdk_screen_get_height (screen) * 6 / 10);
+				     width * 7 / 10,
+				     height * 6 / 10);
 
 	gth_iviewer_zoom_to_fit (GTH_IVIEWER (data->image_selector));
 	gtk_widget_grab_focus (data->image_selector);
