@@ -29,6 +29,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-url.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -43,16 +44,19 @@
 #include "pixbuf-utils.h"
 #include "gconf-utils.h"
 #include "gth-browser.h"
+#include "gth-utils.h"
 #include "glib-utils.h"
 
-static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3, 4 };
-static GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME, 
-					      GTH_SORT_METHOD_BY_PATH, 
-					      GTH_SORT_METHOD_BY_SIZE, 
+static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3, 4, 5 };
+static GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME,
+					      GTH_SORT_METHOD_BY_PATH,
+					      GTH_SORT_METHOD_BY_SIZE,
 					      GTH_SORT_METHOD_BY_TIME,
+					      GTH_SORT_METHOD_BY_EXIF_DATE,
+					      GTH_SORT_METHOD_BY_COMMENT,
 					      GTH_SORT_METHOD_MANUAL};
-static int           idx_to_resize_width[] = { 320, 640, 800, 1024, 1280 };
-static int           idx_to_resize_height[] = { 200, 480, 600, 768, 960 };
+static int           idx_to_resize_width[] = { 320, 320, 640, 640, 800, 800, 1024, 1024, 1280, 1280 };
+static int           idx_to_resize_height[] = { 200, 320, 480, 640, 600, 800, 768, 1024, 960, 1280 };
 
 
 #define str_void(x) (((x) == NULL) || (*(x) == 0))
@@ -83,6 +87,9 @@ typedef struct {
 
 	GtkWidget          *wa_rows_spinbutton;
 	GtkWidget          *wa_cols_spinbutton;
+	GtkWidget          *wa_single_index_checkbutton;
+	GtkWidget          *wa_rows_hbox;
+	GtkWidget          *wa_cols_hbox;
 	GtkWidget          *wa_sort_images_combobox;
 	GtkWidget          *wa_reverse_order_checkbutton;
 
@@ -100,7 +107,7 @@ typedef struct {
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget  *widget, 
+destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	g_object_unref (data->gui);
@@ -110,24 +117,31 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
+/* called when the "help" button is clicked. */
+static void
+help_cb (GtkWidget  *widget,
+         DialogData *data)
+{
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-web-album");
+}
+
+
 static void
 export (GtkWidget  *widget,
 	DialogData *data)
 {
 	CatalogWebExporter *exporter = data->exporter;
 	char               *location;
-	char               *esc_path, *path;
+	char               *path;
 	char               *theme, *index_file;
 	const char         *header;
 	const char         *footer;
 
 	/* Save options. */
 
-	esc_path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->wa_destination_filechooserbutton));
-	path = gnome_vfs_unescape_string (esc_path, "");
+	path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->wa_destination_filechooserbutton));
 	location = remove_ending_separator (path);
 	g_free (path);
-	g_free (esc_path);
 
 	eel_gconf_set_path (PREF_WEB_ALBUM_DESTINATION, location);
 
@@ -146,6 +160,8 @@ export (GtkWidget  *widget,
 
 	eel_gconf_set_integer (PREF_WEB_ALBUM_COLUMNS, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->wa_cols_spinbutton)));
 
+	eel_gconf_set_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_single_index_checkbutton)));
+
 	pref_set_web_album_sort_order (idx_to_sort_method [gtk_combo_box_get_active (GTK_COMBO_BOX (data->wa_sort_images_combobox))]);
 
 	eel_gconf_set_boolean (PREF_WEB_ALBUM_REVERSE, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_reverse_order_checkbutton)));
@@ -180,13 +196,14 @@ export (GtkWidget  *widget,
 
 	catalog_web_exporter_set_copy_images (exporter, eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES, FALSE));
 
-	catalog_web_exporter_set_resize_images (exporter, 
+	catalog_web_exporter_set_resize_images (exporter,
 						eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES, FALSE),
 						idx_to_resize_width[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))],
 						idx_to_resize_height[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))]);
 
 	catalog_web_exporter_set_row_col (exporter, eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS, 4), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS, 4));
-	
+	catalog_web_exporter_set_single_index (exporter, eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
+
 	catalog_web_exporter_set_sorted (exporter, pref_get_web_album_sort_order (), eel_gconf_get_boolean (PREF_WEB_ALBUM_REVERSE, FALSE));
 	catalog_web_exporter_set_header (exporter, header);
 	catalog_web_exporter_set_footer (exporter, footer);
@@ -195,7 +212,7 @@ export (GtkWidget  *widget,
 	g_free (location);
 	g_free (theme);
 	g_free (index_file);
-	
+
 	/* Export. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->progress_dialog),
@@ -246,20 +263,22 @@ static void show_album_theme_cb (GtkWidget *widget, DialogData *data);
 
 
 static int
-get_idx_from_resize_width (int width)
+get_idx_from_size (int width, int height)
 {
+	int idx;
 	if (width == 320)
-		return 0;
+		idx = 0;
 	else if (width == 640)
-		return 1;
+		idx = 1;
 	else if (width == 800)
-		return 2;
+		idx = 2;
 	else if (width == 1024)
-		return 3;
+		idx = 3;
 	else if (width == 1280)
-		return 4;
+		idx = 4;
 	else
-		return 1;
+		idx = 1;
+	return 2 * idx + (width == height? 1 : 0);
 }
 
 
@@ -279,6 +298,15 @@ resize_image_toggled_cb (GtkToggleButton *button,
 }
 
 
+static void
+single_index_toggled_cb (GtkToggleButton *button,
+			 DialogData      *data)
+{
+	gtk_widget_set_sensitive (data->wa_rows_hbox, !gtk_toggle_button_get_active (button));
+}
+
+
+
 static gboolean
 theme_present (const char *theme_name,
 	       const char *theme_dir)
@@ -292,13 +320,13 @@ theme_present (const char *theme_name,
 		return FALSE;
 
 	if (theme_dir != NULL)
-		result = gnome_vfs_directory_list_load (&file_list, 
-							theme_dir, 
+		result = gnome_vfs_directory_list_load (&file_list,
+							theme_dir,
 							GNOME_VFS_FILE_INFO_DEFAULT);
 	else
 		result = GNOME_VFS_ERROR_NOT_A_DIRECTORY;
-	
-	if (result == GNOME_VFS_OK) 
+
+	if (result == GNOME_VFS_OK)
 		for (scan = file_list; scan; scan = scan->next) {
 			GnomeVFSFileInfo *info = scan->data;
 
@@ -358,20 +386,19 @@ get_default_theme (void)
 void
 dlg_web_exporter (GthBrowser *browser)
 {
-	DialogData   *data;
-	GtkWidget    *btn_cancel;
-	GtkWidget    *btn_help;
-	GList        *list;
-	char         *svalue;
-	char         *esc_uri;
-	gboolean      reorderable;
-	int           idx;
+	DialogData *data;
+	GtkWidget  *btn_cancel;
+	GtkWidget  *btn_help;
+	GList      *list;
+	char       *svalue;
+	gboolean    reorderable;
+	int         idx;
 
 	data = g_new0 (DialogData, 1);
 
 	data->browser = browser;
-	
-	list = gth_window_get_file_list_selection (GTH_WINDOW (browser));
+
+	list = gth_window_get_file_list_selection_as_fd (GTH_WINDOW (browser));
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		g_free (data);
@@ -381,8 +408,7 @@ dlg_web_exporter (GthBrowser *browser)
 	reorderable = gth_file_view_get_reorderable (gth_browser_get_file_view (browser));
 
 	data->exporter = catalog_web_exporter_new (GTH_WINDOW (browser), list);
-	g_list_foreach (list, (GFunc) g_free, NULL);
-	g_list_free (list);
+	file_data_list_free (list);
 
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_EXPORTER_FILE, NULL, NULL);
         if (!data->gui) {
@@ -393,7 +419,7 @@ dlg_web_exporter (GthBrowser *browser)
         }
 
 	/* Get the widgets. */
-	
+
 	data->dialog = glade_xml_get_widget (data->gui, "web_album_dialog");
 	data->wa_destination_filechooserbutton = glade_xml_get_widget (data->gui, "wa_destination_filechooserbutton");
 	data->wa_index_file_entry = glade_xml_get_widget (data->gui, "wa_index_file_entry");
@@ -406,6 +432,9 @@ dlg_web_exporter (GthBrowser *browser)
 
 	data->wa_rows_spinbutton = glade_xml_get_widget (data->gui, "wa_rows_spinbutton");
 	data->wa_cols_spinbutton = glade_xml_get_widget (data->gui, "wa_cols_spinbutton");
+	data->wa_single_index_checkbutton = glade_xml_get_widget (data->gui, "wa_single_index_checkbutton");
+	data->wa_rows_hbox = glade_xml_get_widget (data->gui, "wa_rows_hbox");
+	data->wa_cols_hbox = glade_xml_get_widget (data->gui, "wa_cols_hbox");
 	data->wa_sort_images_combobox = glade_xml_get_widget (data->gui, "wa_sort_images_combobox");
 	data->wa_reverse_order_checkbutton = glade_xml_get_widget (data->gui, "wa_reverse_order_checkbutton");
 
@@ -430,7 +459,7 @@ dlg_web_exporter (GthBrowser *browser)
 	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_INDEX_FILE, "index.html");
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->wa_index_file_entry), svalue);
 	g_free (svalue);
-	
+
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_copy_images_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES, FALSE));
 
 	gtk_widget_set_sensitive (data->wa_resize_images_hbox, eel_gconf_get_boolean (PREF_WEB_ALBUM_COPY_IMAGES, FALSE));
@@ -439,12 +468,16 @@ dlg_web_exporter (GthBrowser *browser)
 
 	gtk_widget_set_sensitive (data->wa_resize_images_options_hbox, eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES, FALSE));
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu), get_idx_from_resize_width (eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_WIDTH, 640)));
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu), get_idx_from_size (eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_WIDTH, 640), eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_HEIGHT, 480)));
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_rows_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS, 4));
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_cols_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS, 4));
 
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_single_index_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
+
+	gtk_widget_set_sensitive (data->wa_rows_hbox, !eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
+
 	/**/
 
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
@@ -452,7 +485,11 @@ dlg_web_exporter (GthBrowser *browser)
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
 				   _("by size"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
-				   _("by modified time"));
+				   _("by file modified time"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
+				   _("by Exif DateTime tag"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
+                                   _("by comment"));
 	if (reorderable)
 		gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
 					   _("manual order"));
@@ -485,64 +522,69 @@ dlg_web_exporter (GthBrowser *browser)
 
 	svalue = eel_gconf_get_path (PREF_WEB_ALBUM_DESTINATION, NULL);
 	if (svalue == NULL)
-		esc_uri = gnome_vfs_escape_host_and_path_string (g_get_home_dir ());
-	else
-		esc_uri = gnome_vfs_escape_host_and_path_string (svalue);
-	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->wa_destination_filechooserbutton), esc_uri);
-	g_free (esc_uri);
+		svalue = g_strdup (g_get_home_dir ());
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (data->wa_destination_filechooserbutton), svalue);
 	g_free (svalue);
 
 	/* Signals. */
 
-	g_signal_connect (G_OBJECT (data->dialog), 
+	g_signal_connect (G_OBJECT (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect_swapped (G_OBJECT (btn_cancel), 
+        g_signal_connect (G_OBJECT (btn_help),
+                          "clicked",
+                          G_CALLBACK (help_cb),
+                          data);
+	g_signal_connect_swapped (G_OBJECT (btn_cancel),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
-	g_signal_connect (G_OBJECT (data->btn_ok), 
+	g_signal_connect (G_OBJECT (data->btn_ok),
 			  "clicked",
 			  G_CALLBACK (export),
 			  data);
-
-	g_signal_connect (G_OBJECT (data->wa_select_theme_button), 
+	g_signal_connect (G_OBJECT (data->wa_select_theme_button),
 			  "clicked",
 			  G_CALLBACK (show_album_theme_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->wa_copy_images_checkbutton), 
+	g_signal_connect (G_OBJECT (data->wa_copy_images_checkbutton),
 			  "toggled",
 			  G_CALLBACK (copy_image_toggled_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->wa_resize_images_checkbutton), 
+	g_signal_connect (G_OBJECT (data->wa_resize_images_checkbutton),
 			  "toggled",
 			  G_CALLBACK (resize_image_toggled_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->exporter), 
+	g_signal_connect (G_OBJECT (data->wa_single_index_checkbutton),
+			  "toggled",
+			  G_CALLBACK (single_index_toggled_cb),
+			  data);
+
+	g_signal_connect (G_OBJECT (data->exporter),
 			  "web_exporter_done",
 			  G_CALLBACK (export_done),
 			  data);
-	g_signal_connect (G_OBJECT (data->exporter), 
+	g_signal_connect (G_OBJECT (data->exporter),
 			  "web_exporter_progress",
 			  G_CALLBACK (export_progress),
 			  data);
-	g_signal_connect (G_OBJECT (data->exporter), 
+	g_signal_connect (G_OBJECT (data->exporter),
 			  "web_exporter_info",
 			  G_CALLBACK (export_info),
 			  data);
-	g_signal_connect (G_OBJECT (data->exporter), 
+	g_signal_connect (G_OBJECT (data->exporter),
 			  "web_exporter_start_copying",
 			  G_CALLBACK (export_start_copying),
 			  data);
 
-	g_signal_connect_swapped (G_OBJECT (data->progress_dialog), 
+	g_signal_connect_swapped (G_OBJECT (data->progress_dialog),
 				  "delete_event",
 				  G_CALLBACK (catalog_web_exporter_interrupt),
 				  data->exporter);
-	g_signal_connect_swapped (G_OBJECT (data->progress_cancel), 
+	g_signal_connect_swapped (G_OBJECT (data->progress_cancel),
 				  "clicked",
 				  G_CALLBACK (catalog_web_exporter_interrupt),
 				  data->exporter);
@@ -550,12 +592,12 @@ dlg_web_exporter (GthBrowser *browser)
 	/* Run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (browser));
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show_all (data->dialog);
 }
 
 
-
+
 
 typedef struct {
 	DialogData         *data;
@@ -586,7 +628,7 @@ enum {
 
 /* called when the main dialog is closed. */
 static void
-theme_dialog_destroy_cb (GtkWidget       *widget, 
+theme_dialog_destroy_cb (GtkWidget       *widget,
 			 ThemeDialogData *tdata)
 {
 	g_object_unref (tdata->gui);
@@ -595,7 +637,7 @@ theme_dialog_destroy_cb (GtkWidget       *widget,
 
 
 static void
-theme_dialog__ok_clicked (GtkWidget       *widget, 
+theme_dialog__ok_clicked (GtkWidget       *widget,
 			  ThemeDialogData *tdata)
 {
 	GtkTreeSelection *selection;
@@ -631,13 +673,13 @@ add_theme_dir (ThemeDialogData *tdata,
 	debug (DEBUG_INFO, "theme dir: %s", theme_dir);
 
 	if (theme_dir != NULL)
-		result = gnome_vfs_directory_list_load (&file_list, 
-							theme_dir, 
+		result = gnome_vfs_directory_list_load (&file_list,
+							theme_dir,
 							GNOME_VFS_FILE_INFO_DEFAULT);
 	else
 		result = GNOME_VFS_ERROR_NOT_A_DIRECTORY;
-	
-	if (result == GNOME_VFS_OK) 
+
+	if (result == GNOME_VFS_OK)
 		for (scan = file_list; scan; scan = scan->next) {
 			GnomeVFSFileInfo *info = scan->data;
 			char             *utf8_name;
@@ -650,13 +692,13 @@ add_theme_dir (ThemeDialogData *tdata,
 			    || (strcmp (info->name, "..") == 0))
 				continue;
 
-			utf8_name = g_filename_display_name (info->name);
-			
+			utf8_name = gnome_vfs_unescape_string_for_display (info->name);
+
 			gtk_list_store_append (tdata->list_store, &iter);
 			gtk_list_store_set (tdata->list_store, &iter,
 					    THEME_NAME_COLUMN, utf8_name,
 					    -1);
-			
+
 			g_free (utf8_name);
 		}
 
@@ -694,13 +736,13 @@ load_themes (ThemeDialogData *tdata)
 	model = GTK_TREE_MODEL (tdata->list_store);
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tdata->wat_theme_treeview));
 	theme_name = gtk_entry_get_text (GTK_ENTRY (tdata->data->wa_theme_combo_entry));
-	if (! gtk_tree_model_get_iter_first (model, &iter)) 
+	if (! gtk_tree_model_get_iter_first (model, &iter))
 		return;
 
 	do {
 		char *utf8_name;
-		gtk_tree_model_get (model, &iter, 
-				    THEME_NAME_COLUMN, &utf8_name, 
+		gtk_tree_model_get (model, &iter,
+				    THEME_NAME_COLUMN, &utf8_name,
 				    -1);
 		if (strcmp (utf8_name, theme_name) == 0)
 			gtk_tree_selection_select_iter (selection, &iter);
@@ -723,11 +765,10 @@ ensure_local_theme_dir_exists (void)
 {
 	char *theme_dir;
 
-	theme_dir = g_build_path (G_DIR_SEPARATOR_S,
-				  get_home_uri (),
-				  ".gnome2",
-				  "gthumb/albumthemes",
-				  NULL);
+	theme_dir = build_uri (get_home_uri (),
+			       ".gnome2",
+			       "gthumb/albumthemes",
+			       NULL);
 
 	dir_make (theme_dir, 0700);
 
@@ -741,42 +782,40 @@ install_theme__ok_cb (GtkDialog  *file_sel,
 		      gpointer    data)
 {
 	ThemeDialogData  *tdata;
-	char             *theme_archive, *e_theme_archive;
+	char             *theme_archive;
 	char             *command_line = NULL;
 	GError           *err = NULL;
 
 	tdata = g_object_get_data (G_OBJECT (file_sel), "theme_dialog_data");
-	e_theme_archive = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
+	theme_archive = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
+
 	gtk_widget_destroy (GTK_WIDGET (file_sel));
 
-	if (e_theme_archive == NULL)
+	if (theme_archive == NULL)
 		return;
 
 	/**/
 
-	theme_archive = gnome_vfs_unescape_string (e_theme_archive, "");
-	g_free (e_theme_archive);
-
 	ensure_local_theme_dir_exists ();
-            
+
 	if (file_extension_is (theme_archive, ".tar.gz")
 	    || file_extension_is (theme_archive, ".tgz"))
 		command_line = g_strdup_printf ("tar -C %s%s -zxf %s",
 						g_get_home_dir (),
 						"/.gnome2/gthumb/albumthemes",
 						theme_archive);
-	
+
 	else if (file_extension_is (theme_archive, ".tar.bz2"))
 		command_line = g_strdup_printf ("tar -C %s%s -xf %s --use-compress-program bzip2",
 						g_get_home_dir (),
 						"/.gnome2/gthumb/albumthemes",
 						theme_archive);
 
-	if ((command_line != NULL) 
+	if ((command_line != NULL)
 	    && ! g_spawn_command_line_sync (command_line, NULL, NULL, NULL, &err)
 	    && (err != NULL))
 		_gtk_error_dialog_from_gerror_run (NULL, &err);
-	
+
 	g_free (command_line);
 	g_free (theme_archive);
 
@@ -792,15 +831,15 @@ install_theme_response_cb (GtkDialog  *file_sel,
 			   int         button_number,
 			   gpointer    userdata)
 {
-	if (button_number == GTK_RESPONSE_ACCEPT) 
+	if (button_number == GTK_RESPONSE_ACCEPT)
 		install_theme__ok_cb (file_sel, button_number, userdata);
-	 else 
+	 else
 		gtk_widget_destroy (GTK_WIDGET (file_sel));
 }
 
 
 static void
-theme_dialog__install_theme_clicked (GtkWidget       *widget, 
+theme_dialog__install_theme_clicked (GtkWidget       *widget,
 				     ThemeDialogData *tdata)
 {
 	GtkWidget *file_sel;
@@ -810,6 +849,10 @@ theme_dialog__install_theme_clicked (GtkWidget       *widget,
 						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 						NULL);
+
+	/* Permit VFS URIs */
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
+
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 	g_object_set_data (G_OBJECT (file_sel), "theme_dialog_data", tdata);
 
@@ -828,13 +871,13 @@ theme_dialog__install_theme_clicked (GtkWidget       *widget,
 
 
 static void
-theme_dialog__go_to_folder_clicked (GtkWidget       *widget, 
+theme_dialog__go_to_folder_clicked (GtkWidget       *widget,
 				    ThemeDialogData *tdata)
 {
 	char         *path;
 	GError       *err;
 
-	path = g_strdup_printf ("file://%s/.gnome2/gthumb/albumthemes", 
+	path = g_strdup_printf ("file://%s/.gnome2/gthumb/albumthemes",
 			       g_get_home_dir ());
 
 	ensure_dir_exists (path, 0775);
@@ -849,7 +892,7 @@ theme_dialog__go_to_folder_clicked (GtkWidget       *widget,
 /* called when an item of the catalog list is selected. */
 static void
 theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
-			      gpointer p)
+			      gpointer          p)
 {
 	ThemeDialogData  *tdata = p;
 	gboolean          theme_selected;
@@ -858,8 +901,8 @@ theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
 	char             *theme, *path;
 
 	theme_selected = gtk_tree_selection_get_selected (selection, NULL, &iter);
-	
-	if (!theme_selected) 
+
+	if (!theme_selected)
 		return;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (tdata->list_store),
@@ -867,7 +910,7 @@ theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
 			    THEME_NAME_COLUMN, &utf8_name,
 			    -1);
 
-	theme = g_filename_from_utf8 (utf8_name, -1, 0, 0, 0);
+	theme = gnome_vfs_escape_string (utf8_name);
 	path = g_build_path (G_DIR_SEPARATOR_S,
 			     g_get_home_dir (),
 			     ".gnome2",
@@ -885,12 +928,12 @@ theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
 	}
 
 	if (path_is_dir (path)) {
-		char *filename = g_build_path (G_DIR_SEPARATOR_S,
-					       path, 
-					       "preview.png", 
-					       NULL);
-		if (path_is_file (filename)) {
-			GdkPixbuf *image = gdk_pixbuf_new_from_file (filename, NULL);
+		char      *filename;
+		GdkPixbuf *image = NULL;
+					       
+		filename = build_uri (path, "preview.png", NULL);
+		if (path_is_file (filename)
+		    && ((image = gdk_pixbuf_new_from_file (filename, NULL)) != NULL)) {
 			int        w = gdk_pixbuf_get_width (image);
 			int        h = gdk_pixbuf_get_height (image);
 			if (scale_keepping_ratio (&w, &h, MAX_PREVIEW_SIZE, MAX_PREVIEW_SIZE)) {
@@ -899,11 +942,13 @@ theme_dialog__sel_changed_cb (GtkTreeSelection *selection,
 				g_object_unref (tmp);
 			}
 			gtk_image_set_from_pixbuf (GTK_IMAGE (tdata->wat_preview_image), image);
-			g_object_unref (image);
-
-		} else
+		} 
+		else
 			gtk_image_set_from_stock (GTK_IMAGE (tdata->wat_preview_image), GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
+
 		g_free (filename);
+		if (image != NULL)
+			g_object_unref (image);
 	}
 
 	g_free (utf8_name);
@@ -951,15 +996,15 @@ show_album_theme_cb (GtkWidget  *widget,
 
 	/* Signals. */
 
-	g_signal_connect (G_OBJECT (tdata->dialog), 
+	g_signal_connect (G_OBJECT (tdata->dialog),
 			  "destroy",
 			  G_CALLBACK (theme_dialog_destroy_cb),
 			  tdata);
-	g_signal_connect_swapped (G_OBJECT (tdata->wat_cancel_button), 
+	g_signal_connect_swapped (G_OBJECT (tdata->wat_cancel_button),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (tdata->dialog));
-	g_signal_connect (G_OBJECT (tdata->wat_ok_button), 
+	g_signal_connect (G_OBJECT (tdata->wat_ok_button),
 			  "clicked",
 			  G_CALLBACK (theme_dialog__ok_clicked),
 			  tdata);
@@ -978,15 +1023,15 @@ show_album_theme_cb (GtkWidget  *widget,
 			  "clicked",
 			  G_CALLBACK (theme_dialog__install_theme_clicked),
 			  tdata);
-	g_signal_connect (G_OBJECT (tdata->wat_go_to_folder_button), 
+	g_signal_connect (G_OBJECT (tdata->wat_go_to_folder_button),
 			  "clicked",
 			  G_CALLBACK (theme_dialog__go_to_folder_clicked),
 			  tdata);
-	g_signal_connect (G_OBJECT (tdata->wat_thumbnail_caption_button), 
+	g_signal_connect (G_OBJECT (tdata->wat_thumbnail_caption_button),
 			  "clicked",
 			  G_CALLBACK (show_thumbnail_caption_dialog_cb),
 			  tdata);
-	g_signal_connect (G_OBJECT (tdata->wat_image_caption_button), 
+	g_signal_connect (G_OBJECT (tdata->wat_image_caption_button),
 			  "clicked",
 			  G_CALLBACK (show_image_caption_dialog_cb),
 			  tdata);
@@ -1007,7 +1052,7 @@ show_album_theme_cb (GtkWidget  *widget,
                                              "text", THEME_NAME_COLUMN,
                                              NULL);
 
-        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);        
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, THEME_NAME_COLUMN);
         gtk_tree_view_append_column (GTK_TREE_VIEW (tdata->wat_theme_treeview), column);
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (tdata->list_store), THEME_NAME_COLUMN, GTK_SORT_ASCENDING);
@@ -1016,16 +1061,14 @@ show_album_theme_cb (GtkWidget  *widget,
 
 	/* Run dialog. */
 
-	gtk_widget_grab_focus (tdata->wat_theme_treeview); 
+	gtk_widget_grab_focus (tdata->wat_theme_treeview);
 
 	gtk_window_set_transient_for (GTK_WINDOW (tdata->dialog), GTK_WINDOW (data->dialog));
-	gtk_window_set_modal (GTK_WINDOW (tdata->dialog), FALSE);
+	gtk_window_set_modal (GTK_WINDOW (tdata->dialog), TRUE);
 	gtk_widget_show_all (tdata->dialog);
 }
 
 
-
-
 typedef struct {
 	DialogData         *data;
 	GthBrowser         *browser;
@@ -1034,6 +1077,8 @@ typedef struct {
 	GtkWidget          *dialog;
 
 	GtkWidget          *c_comment_checkbutton;
+	GtkWidget          *c_place_checkbutton;
+	GtkWidget          *c_date_time_checkbutton;
 	GtkWidget          *c_imagedim_checkbutton;
 	GtkWidget          *c_filename_checkbutton;
 	GtkWidget          *c_filesize_checkbutton;
@@ -1053,7 +1098,7 @@ typedef struct {
 
 /* called when the dialog is closed. */
 static void
-caption_dialog_destroy_cb (GtkWidget         *widget, 
+caption_dialog_destroy_cb (GtkWidget         *widget,
 			   CaptionDialogData *cdata)
 {
 	g_object_unref (cdata->gui);
@@ -1062,7 +1107,7 @@ caption_dialog_destroy_cb (GtkWidget         *widget,
 
 
 static void
-caption_dialog__ok_clicked (GtkWidget         *widget, 
+caption_dialog__ok_clicked (GtkWidget         *widget,
 			    CaptionDialogData *cdata)
 {
 	const char       *gconf_key;
@@ -1070,6 +1115,10 @@ caption_dialog__ok_clicked (GtkWidget         *widget,
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton)))
 		caption |= GTH_CAPTION_COMMENT;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_place_checkbutton)))
+		caption |= GTH_CAPTION_PLACE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_date_time_checkbutton)))
+		caption |= GTH_CAPTION_DATE_TIME;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton)))
 		caption |= GTH_CAPTION_IMAGE_DIM;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_filename_checkbutton)))
@@ -1115,12 +1164,8 @@ show_caption_dialog_cb (GtkWidget       *widget,
 	CaptionDialogData *cdata;
 	GtkWidget         *ok_button;
 	GtkWidget         *cancel_button;
-#ifdef HAVE_LIBEXIF
 	const char        *gconf_key;
 	GthCaptionFields   caption = 0;
-#else
-	GtkWidget         *vbox;
-#endif
 	cdata = g_new (CaptionDialogData, 1);
 
 	cdata->data = tdata->data;
@@ -1138,6 +1183,8 @@ show_caption_dialog_cb (GtkWidget       *widget,
 
 	cdata->dialog = glade_xml_get_widget (cdata->gui, "caption_dialog");
 	cdata->c_comment_checkbutton = glade_xml_get_widget (cdata->gui, "c_comment_checkbutton");
+	cdata->c_place_checkbutton = glade_xml_get_widget (cdata->gui, "c_place_checkbutton");
+	cdata->c_date_time_checkbutton = glade_xml_get_widget (cdata->gui, "c_date_time_checkbutton");
 	cdata->c_imagedim_checkbutton = glade_xml_get_widget (cdata->gui, "c_imagedim_checkbutton");
 	cdata->c_filename_checkbutton = glade_xml_get_widget (cdata->gui, "c_filename_checkbutton");
 	cdata->c_filesize_checkbutton = glade_xml_get_widget (cdata->gui, "c_filesize_checkbutton");
@@ -1155,31 +1202,33 @@ show_caption_dialog_cb (GtkWidget       *widget,
 
 	/* Signals. */
 
-	g_signal_connect (G_OBJECT (cdata->dialog), 
+	g_signal_connect (G_OBJECT (cdata->dialog),
 			  "destroy",
 			  G_CALLBACK (caption_dialog_destroy_cb),
 			  cdata);
-	g_signal_connect_swapped (G_OBJECT (cancel_button), 
+	g_signal_connect_swapped (G_OBJECT (cancel_button),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (cdata->dialog));
-	g_signal_connect (G_OBJECT (ok_button), 
+	g_signal_connect (G_OBJECT (ok_button),
 			  "clicked",
 			  G_CALLBACK (caption_dialog__ok_clicked),
 			  cdata);
 
 	/* Set widgets data. */
 
-#ifdef HAVE_LIBEXIF
-
-	if (cdata->thumbnail_caption) 
+	if (cdata->thumbnail_caption)
 		gconf_key = PREF_WEB_ALBUM_INDEX_CAPTION;
-	 else 
+	 else
 		gconf_key = PREF_WEB_ALBUM_IMAGE_CAPTION;
 	caption = eel_gconf_get_integer (gconf_key, 0);
 
 	if (caption & GTH_CAPTION_COMMENT)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_PLACE)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_place_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_DATE_TIME)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_date_time_checkbutton), TRUE);
 	if (caption & GTH_CAPTION_IMAGE_DIM)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton), TRUE);
 	if (caption & GTH_CAPTION_FILE_NAME)
@@ -1203,10 +1252,6 @@ show_caption_dialog_cb (GtkWidget       *widget,
 	if (caption & GTH_CAPTION_EXIF_CAMERA_MODEL)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_camera_model_checkbutton), TRUE);
 
-#else /* ! HAVE_LIBEXIF */
-	vbox = glade_xml_get_widget (cdata->gui, "c_exif_data_vbox");
-	gtk_widget_hide (vbox);
-#endif /* ! HAVE_LIBEXIF */
 
 	/* Run dialog. */
 
