@@ -256,6 +256,7 @@ combo_changed_cb (GtkComboBox *widget,
 	GthLocation *loc = user_data;
 	GtkTreeIter  iter;
 	char        *path = NULL;
+	char        *uri = NULL;
 	int          item_type = ITEM_TYPE_NONE;
 
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (loc->priv->combo), &iter))
@@ -265,6 +266,7 @@ combo_changed_cb (GtkComboBox *widget,
 			    &iter,
 			    PATH_COLUMN, &path,
 			    TYPE_COLUMN, &item_type,
+			    PATH_COLUMN, &uri,
 			    -1);
 
 	if (item_type == ITEM_TYPE_OPEN_LOCATION) {
@@ -278,8 +280,12 @@ combo_changed_cb (GtkComboBox *widget,
 	if (path == NULL)
 		return;
 
+	g_free (loc->priv->uri);
+	loc->priv->uri = add_scheme_if_absent (uri);
+
 	g_signal_emit (G_OBJECT (loc), gth_location_signals[CHANGED], 0, path);
 	g_free (path);
+	g_free (uri);
 }
 
 
@@ -633,9 +639,6 @@ update_drives (GthLocation *loc)
 
 	clear_items (loc, ITEM_TYPE_DRIVE);
 
-	if (g_list_length (loc->priv->drives) == 0)
-		return;
-
 	if (!get_first_separator_pos (loc, &pos))
 		return;
 
@@ -644,12 +647,15 @@ update_drives (GthLocation *loc)
 	/* Home, File System */
 
 	insert_drive_from_uri (loc, get_home_uri (), pos++);
-	if (!loc->priv->folders_only)
-		insert_drive_from_uri (loc, "catalog://", pos++);
 	insert_drive_from_uri (loc, "file://", pos++);
+	if (!loc->priv->folders_only)
+		insert_drive_from_uri (loc, "catalog:///", pos++);
 
 	/* Other drives */
 
+	if (g_list_length (loc->priv->drives) == 0)
+		return;
+
 	for (scan = loc->priv->drives; scan; scan = scan->next) {
 		GnomeVFSDrive *drive = scan->data;
 		char          *uri;
@@ -780,16 +786,18 @@ update_uri (GthLocation *loc,
 	home_uri = g_strconcat ("file://", g_get_home_dir(), NULL);
 	uri = g_strdup (loc->priv->uri);
 
-	if (loc->priv->catalog_uri)
+	if (loc->priv->catalog_uri) {
 		base_uri = g_strconcat (CATALOG_PREFIX,
 					g_get_home_dir(),
 					"/",
 					RC_CATALOG_DIR,
 					NULL);
+	}
 	else {
 		volume = get_volume_from_uri (loc, uri);
-		if (volume == NULL)
+		if (volume == NULL) {
 			base_uri = get_base_uri (uri);
+		}
 		else {
 			base_uri = gnome_vfs_volume_get_activation_uri (volume);
 			drive = gnome_vfs_volume_get_drive (volume);
@@ -809,11 +817,13 @@ update_uri (GthLocation *loc,
 		}
 
 		if (same_uri (uri, base_uri)) {
-			if (loc->priv->catalog_uri)
+			if (loc->priv->catalog_uri) {
 				uri_name = g_strdup (_("Catalogs"));
+			}
 			else {
-				if (volume != NULL)
+				if (volume != NULL) {
 					uri_name = gnome_vfs_volume_get_display_name (volume);
+				}
 				else {
 					if (uri_scheme_is_file (base_uri))
 						uri_name = g_strdup (_("File System"));
@@ -821,7 +831,8 @@ update_uri (GthLocation *loc,
 						uri_name = g_strdup (base_uri);
 				}
 			}
-		} else {
+		} 
+		else {
 			if (same_uri (uri, home_uri))
 				uri_name = g_strdup (_("Home"));
 			else
@@ -868,7 +879,7 @@ gth_location_set_uri (GthLocation *loc,
 		      gboolean     reset_history)
 {
 	g_free (loc->priv->uri);
-	loc->priv->uri = get_uri_from_path (uri);
+	loc->priv->uri = add_scheme_if_absent (uri);
 	update_uri (loc, reset_history);
 }
 
