@@ -72,22 +72,22 @@ static GtkToggleActionEntry gth_viewer_action_toggle_entries[] = {
 	{ "View_Toolbar", NULL,
 	  N_("_Toolbar"), NULL,
 	  N_("View or hide the toolbar of this window"),
-	  G_CALLBACK (gth_viewer_activate_action_view_toolbar), 
+	  G_CALLBACK (gth_viewer_activate_action_view_toolbar),
 	  TRUE },
 	{ "View_Statusbar", NULL,
 	  N_("_Statusbar"), NULL,
 	  N_("View or hide the statusbar of this window"),
-	  G_CALLBACK (gth_viewer_activate_action_view_statusbar), 
+	  G_CALLBACK (gth_viewer_activate_action_view_statusbar),
 	  TRUE },
-	{ "View_ShowInfo", NULL,
-	  N_("Image _Comment"), NULL,
-	  N_("View image comment"),
-	  G_CALLBACK (gth_viewer_activate_action_view_show_info), 
+	{ "View_ShowMetadata", GTK_STOCK_PROPERTIES,
+	  NULL, NULL,
+	  N_("View image properties"),
+	  G_CALLBACK (gth_viewer_activate_action_view_show_info),
 	  TRUE },
 	{ "View_SingleWindow", NULL,
 	  N_("_Single Window"), NULL,
 	  N_("Reuse this window to view other images"),
-	  G_CALLBACK (gth_viewer_activate_action_single_window), 
+	  G_CALLBACK (gth_viewer_activate_action_single_window),
 	  FALSE }
 };
 static guint gth_viewer_action_toggle_entries_size = G_N_ELEMENTS (gth_viewer_action_toggle_entries);
