@@ -23,16 +23,16 @@
 #include <config.h>
 #include <gnome.h>
 #include <glade/glade.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomeui/gnome-ui-init.h>
+#include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <libbonobo.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 
 #include "catalog.h"
 #include "comments.h"
 #include "main.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "gconf-utils.h"
 #include "gtk-utils.h"
 #include "gth-application.h"
@@ -63,40 +63,50 @@ static char*icon_mime_name[ICON_NAMES] = { "gnome-fs-directory",
 					   "gnome-fs-home",
 					   "gnome-fs-desktop" };
 
-GthMonitor  *monitor = NULL;
-char       **file_urls, **dir_urls;
-int          n_file_urls, n_dir_urls;
-int          StartInFullscreen = FALSE;
-int          StartSlideshow = FALSE;
-int          ViewFirstImage = FALSE;
-int          HideSidebar = FALSE;
-gboolean     ExitAll = FALSE;
-char        *ImageToDisplay = NULL;
-gboolean     FirstStart = TRUE;
-gboolean     ImportPhotos = FALSE;
-gboolean     UseViewer = FALSE;
+GthMonitor    *gth_monitor = NULL;
+GList         *file_urls = NULL, *dir_urls = NULL;
+int            n_file_urls, n_dir_urls;
+int            StartInFullscreen = FALSE;
+int            StartSlideshow = FALSE;
+int            ViewFirstImage = FALSE;
+int            HideSidebar = FALSE;
+gboolean       ExitAll = FALSE;
+char          *ImageToDisplay = NULL;
+gboolean       FirstStart = TRUE;
+gboolean       ImportPhotos = FALSE;
+gboolean       UseViewer = FALSE;
 
-static gboolean        view_comline_catalog = FALSE;
-static gboolean        view_single_image = FALSE;
-static GdkPixbuf      *icon_pixbuf[ICON_NAMES] = { 0 };
-static GtkWidget      *first_window = NULL;
-static GnomeIconTheme *icon_theme = NULL;
-static BonoboObject   *gth_application = NULL;
+static gboolean         view_comline_catalog = FALSE;
+static gboolean         view_single_image = FALSE;
+static GdkPixbuf       *icon_pixbuf[ICON_NAMES] = { 0 };
+static GtkWidget       *first_window = NULL;
+static GtkIconTheme    *icon_theme = NULL;
+static GOptionContext  *context;
+static char           **remaining_args;
+static BonoboObject    *gth_application = NULL;
 
-struct poptOption options[] = {
-	{ "fullscreen", 'f', POPT_ARG_NONE, &StartInFullscreen, 0,
+static const GOptionEntry options[] = {
+	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &StartInFullscreen,
 	  N_("Start in fullscreen mode"),
 	  0 },
-	{ "slideshow", 's', POPT_ARG_NONE, &StartSlideshow, 0,
+
+	{ "slideshow", 's', 0, G_OPTION_ARG_NONE, &StartSlideshow,
 	  N_("Automatically start a slideshow"),
 	  0 },
-	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
+
+	{ "import-photos", '\0', 0, G_OPTION_ARG_NONE, &ImportPhotos,
 	  N_("Automatically import digital camera photos"),
 	  0 },
-	{ "viewer", '\0', POPT_ARG_NONE, &UseViewer, 0,
-	  "Use the viewer mode to view single images",
+
+	{ "viewer", '\0', 0, G_OPTION_ARG_NONE, &UseViewer,
+	  N_("Use the viewer mode to view single images"),
 	  0 },
-	{ NULL, '\0', 0, NULL, 0 }
+
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining_args,
+	  NULL,
+	  NULL },
+
+	{ NULL }
 };
 
 
@@ -122,7 +132,7 @@ free_icon_pixbufs (void)
 
 
 static void
-theme_changed_cb (GnomeIconTheme *theme,
+theme_changed_cb (GtkIconTheme *theme,
 		  gpointer        data)
 {
 	free_icon_pixbufs ();
@@ -176,7 +186,7 @@ convert_old_comment (char     *real_file,
 	char *comment_file;
 	char *comment_dir;
 
-	comment_file = comments_get_comment_filename (real_file, TRUE, TRUE);
+	comment_file = comments_get_comment_filename (real_file, TRUE);
 	comment_dir = remove_level_from_path (comment_file);
 	ensure_dir_exists (comment_dir, 0755);
 
@@ -234,9 +244,9 @@ save_session (GnomeClient *client)
 			location = gth_window_get_image_filename (window);
 			if (location == NULL)
 				continue;
-			uri = get_uri_from_path (location);
-
-		} else {
+			uri = add_scheme_if_absent (location);
+		} 
+		else {
 			GthBrowser *browser = (GthBrowser*) window;
 
 			switch (gth_browser_get_sidebar_content (browser)) {
@@ -244,14 +254,14 @@ save_session (GnomeClient *client)
 				location = gth_browser_get_current_directory (browser);
 				if (location == NULL)
 					continue;
-				uri = get_uri_from_path (location);
+				uri = add_scheme_if_absent (location);
 				break;
 
 			case GTH_SIDEBAR_CATALOG_LIST:
 				location = gth_browser_get_current_catalog (browser);
 				if (location == NULL)
 					continue;
-				uri = get_uri_from_path (location);
+				uri = add_scheme_if_absent (location);
 				break;
 
 			default:
@@ -360,27 +370,63 @@ session_is_restored (void)
 }
 
 
+static char *
+get_command_line_catalog_path (void)
+{
+	char *catalog_name_utf8;
+	char *catalog_name;
+	char *catalog_path;
+
+	catalog_name_utf8 = g_strconcat (_("Command Line"),
+					 CATALOG_EXT,
+					 NULL);
+	catalog_name = gnome_vfs_escape_string (catalog_name_utf8);
+	catalog_path = get_catalog_full_path (catalog_name);
+	g_free (catalog_name);
+	g_free (catalog_name_utf8);
+
+	return catalog_path;
+}
+
+
+static void
+reset_command_line_catalog (void)
+{
+	char *catalog_path;
+	char *catalog_uri;
+
+	/* Reset the startup location if it was set to the command
+	 * line catalog. */
+
+	catalog_path = get_command_line_catalog_path ();
+	catalog_uri = g_strconcat ("catalog://", remove_host_from_uri (catalog_path), NULL);
+
+	if (strcmp (catalog_uri, preferences_get_startup_location ()) == 0)
+		preferences_set_startup_location (get_home_uri ());
+
+	g_free (catalog_uri);
+	g_free (catalog_path);
+}
+
+
 /* Initialize application data. */
 static void
-initialize_data (poptContext pctx)
+initialize_data (void)
 {
-	const char **argv;
-	int          argc;
-	char        *current_dir;
-	char        *path;
-	int          i;
+	char *current_dir;
+	char *path, *filename;
+	int   i = 0;
 
 	convert_to_new_comment_system ();
 	create_default_categories_if_needed ();
 
 	eel_gconf_monitor_add ("/apps/gthumb");
 
-	monitor = gth_monitor_new ();
+	gth_monitor = gth_monitor_new ();
 
 	/* Icon theme */
 
-	icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
+	icon_theme = gtk_icon_theme_get_default ();
 	g_signal_connect (icon_theme,
 			  "changed",
 			  G_CALLBACK (theme_changed_cb),
@@ -400,44 +446,23 @@ initialize_data (poptContext pctx)
 
 	/* Parse command line arguments. */
 
-	argv = poptGetArgs (pctx);
-
-	if (argv == NULL)
+	if (remaining_args == NULL) { /* No arguments specified. */
+		reset_command_line_catalog ();
 		return;
-
-	for (argc = 0; argv[argc] != NULL; argc++)
-		;
-
-	if (argc == 0)
-		return;
-
-	file_urls = g_new0 (char*, argc);
-	dir_urls = g_new0 (char*, argc);
-
-	n_file_urls = 0;
-	n_dir_urls = 0;
+	}
 
 	current_dir = g_get_current_dir ();
-	for (i = 0; i < argc; i++) {
-		char     *tmp1, *tmp2;
+	while ((filename = remaining_args[i++]) != NULL) {
+		char     *tmp1 = NULL;
 		gboolean  is_dir;
 
-		if (! g_path_is_absolute (argv[i]))
-			tmp1 = g_strconcat (current_dir, "/", argv[i], NULL);
+		if (uri_has_scheme (filename) || g_path_is_absolute (filename))
+			tmp1 = gnome_vfs_make_uri_from_shell_arg (filename);
 		else
-			tmp1 = g_strdup (argv[i]);
-
-		tmp2 = remove_special_dirs_from_path (tmp1);
-
-		if (! g_path_is_absolute (tmp2))
-			path = g_strconcat (current_dir, "/", tmp2, NULL);
-		else {
-			path = tmp2;
-			tmp2 = NULL;
-		}
+			tmp1 = g_strconcat (current_dir, "/", filename, NULL);
 
+		path = remove_special_dirs_from_path (tmp1);
 		g_free (tmp1);
-		g_free (tmp2);
 
 		if (path_is_dir (path))
 			is_dir = TRUE;
@@ -449,42 +474,42 @@ initialize_data (poptContext pctx)
 		}
 
 		if (is_dir) {
-			dir_urls[n_dir_urls++] = get_uri_from_path (path);
+			dir_urls = g_list_prepend (dir_urls, add_scheme_if_absent (path));
 			g_free (path);
-		} else
-			file_urls[n_file_urls++] = path;
+		} 
+		else
+			file_urls = g_list_prepend (file_urls, path);
 	}
 
-	if ((n_file_urls == 1) && (file_urls[0] != NULL)) {
+	n_file_urls = g_list_length (file_urls);
+	n_dir_urls = g_list_length (dir_urls);
+
+	if (n_file_urls == 1)
 		view_single_image = TRUE;
 
-	} else if (n_file_urls > 1) {
+	if (n_file_urls > 1) {
 		/* Create a catalog with the command line list. */
 		Catalog *catalog;
 		char    *catalog_path;
-		char    *catalog_name, *catalog_name_utf8;
+		GList   *scan;
 
 		catalog = catalog_new ();
-
-		catalog_name_utf8 = g_strconcat (_("Command Line"),
-						 CATALOG_EXT,
-						 NULL);
-		catalog_name = g_filename_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
-		catalog_path = get_catalog_full_path (catalog_name);
-		g_free (catalog_name);
-		g_free (catalog_name_utf8);
-
+		catalog_path = get_command_line_catalog_path ();
 		catalog_set_path (catalog, catalog_path);
 		g_free (catalog_path);
 
-		for (i = 0; i < n_file_urls; i++)
-			catalog_add_item (catalog, file_urls[i]);
+		for (scan = file_urls; scan; scan = scan->next)
+			catalog_add_item (catalog, scan->data);
+
+		catalog->sort_method = GTH_SORT_METHOD_MANUAL;
 
 		catalog_write_to_disk (catalog, NULL);
 		catalog_free (catalog);
 
 		view_comline_catalog = TRUE;
 	}
+	else
+		reset_command_line_catalog ();
 
 	g_free (current_dir);
 }
@@ -496,17 +521,15 @@ release_data (void)
 {
 	if (gth_application != NULL)
                 bonobo_object_unref (gth_application);
-
+                	
 	free_icon_pixbufs ();
 
-	g_object_unref (icon_theme);
-
-	if (monitor != NULL) {
-		g_object_unref (monitor);
-		monitor = NULL;
+	if (gth_monitor != NULL) {
+		g_object_unref (gth_monitor);
+		gth_monitor = NULL;
 	}
 
-	preferences_release ();
+	gthumb_release ();
 	eel_global_client_free ();
 }
 
@@ -595,7 +618,7 @@ prepare_app (void)
 	gboolean                  use_factory = FALSE;
 	CORBA_Environment         env;
 	GNOME_GThumb_Application  app;
-	int                       i;
+	GList                    *scan;
 
 	factory = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application_Factory",
                                                       Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
@@ -614,62 +637,51 @@ prepare_app (void)
 
 	if (ImportPhotos) {
 		if (use_factory)
-			GNOME_GThumb_Application_import_photos (app, &env);
+		 	GNOME_GThumb_Application_import_photos (app, &env);
 		else
 			gth_browser_activate_action_file_camera_import (NULL, NULL);
-
-	} else if (! view_comline_catalog
-	    && (n_dir_urls == 0)
-	    && (n_file_urls == 0)) {
+	} 
+	else if (! view_comline_catalog
+		 && (n_dir_urls == 0)
+		 && (n_file_urls == 0)) 
+	{
 		open_browser_window (NULL, TRUE, use_factory, app, &env);
-
-	} else if (view_single_image) {
-		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE))
-			GNOME_GThumb_Application_load_image (app, file_urls[0], &env);
+	} 
+	else if (view_single_image) {
+		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE)) 
+		{ 
+			GNOME_GThumb_Application_load_image (app, file_urls->data, &env);
+		}
 		else {
 			if (UseViewer)
-				open_viewer_window (file_urls[0], use_factory, app, &env);
+				open_viewer_window (file_urls->data, use_factory, app, &env);
 			else
-				open_browser_window (file_urls[0], TRUE, use_factory, app, &env);
+				open_browser_window (file_urls->data, TRUE, use_factory, app, &env);
 		}
-
-	} else if (view_comline_catalog) {
-		char *catalog_uri;
+	} 
+	else if (view_comline_catalog) {
 		char *catalog_path;
-		char *catalog_name;
+		char *catalog_uri;
 
 		ViewFirstImage = TRUE;
 		HideSidebar = TRUE;
 
-		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
-		catalog_path = get_catalog_full_path (catalog_name);
-		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
+		catalog_path = get_command_line_catalog_path ();
+		catalog_uri = g_strconcat ("catalog://", remove_host_from_uri (catalog_path), NULL);
 
 		open_browser_window (catalog_uri, TRUE, use_factory, app, &env);
 
-		g_free (catalog_name);
 		g_free (catalog_path);
 		g_free (catalog_uri);
 	}
 
-	for (i = 0; i < n_dir_urls; i++) {
+	for (scan = dir_urls; scan; scan = scan->next) {
 		/* Go to the specified directory. */
-		open_browser_window (dir_urls[i], TRUE, use_factory, app, &env);
+		open_browser_window (scan->data, TRUE, use_factory, app, &env);
 	}
 
-	/* Free urls. */
-
-	if (n_file_urls > 0) {
-		for (i = 0; i < n_file_urls; i++)
-			g_free (file_urls[i]);
-		g_free (file_urls);
-	}
-
-	if (n_dir_urls > 0) {
-		for (i = 0; i < n_dir_urls; i++)
-			g_free (dir_urls[i]);
-		g_free (dir_urls);
-	}
+	path_list_free (file_urls);
+	path_list_free (dir_urls);
 
 	/**/
 
@@ -678,7 +690,8 @@ prepare_app (void)
 		CORBA_exception_free (&env);
 		gdk_notify_startup_complete ();
 		exit (0);
-	} else
+	} 
+	else
 		gth_application = gth_application_new (gdk_screen_get_default ());
 }
 
@@ -688,17 +701,26 @@ main (int   argc,
       char *argv[])
 {
 	GnomeProgram *program;
-	GValue        value = { 0 };
-	poptContext   pctx;
+	char         *description;
+
+	if (! g_thread_supported ()) {
+		g_thread_init (NULL);
+		gdk_threads_init ();
+	}
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	description = g_strdup_printf ("- %s", _("View and organize your images"));
+	context = g_option_context_new (description);
+	g_free (description);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("gthumb", VERSION,
 				      LIBGNOMEUI_MODULE,
 				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
 				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
 				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
@@ -706,23 +728,16 @@ main (int   argc,
 				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
 				      NULL);
 
-	if (! g_thread_supported ()) {
-		g_thread_init (NULL);
-		gdk_threads_init ();
-	}
-
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-
-	pctx = g_value_get_pointer (&value);
+	gnome_vfs_init ();				      
+	gnome_authentication_manager_init ();
 	glade_gnome_init ();
 	gthumb_init ();
-	initialize_data (pctx);
-	poptFreeContext (pctx);
+	initialize_data ();
 	prepare_app ();
 
+	gdk_threads_enter ();
 	gtk_main ();
+	gdk_threads_leave ();
 
 	release_data ();
 	return 0;
@@ -735,14 +750,14 @@ main (int   argc,
 void
 all_windows_update_catalog_list (void)
 {
-	gth_monitor_notify_reload_catalogs (monitor);
+	gth_monitor_notify_reload_catalogs (gth_monitor);
 }
 
 
 void
 all_windows_notify_update_bookmarks (void)
 {
-	gth_monitor_notify_update_bookmarks (monitor);
+	gth_monitor_notify_update_bookmarks (gth_monitor);
 }
 
 
@@ -750,7 +765,7 @@ void
 all_windows_notify_cat_files_created (const char *catalog_path,
 				      GList      *list)
 {
-	gth_monitor_notify_update_cat_files (monitor,
+	gth_monitor_notify_update_cat_files (gth_monitor,
 					     catalog_path,
 					     GTH_MONITOR_EVENT_CREATED,
 					     list);
@@ -761,7 +776,7 @@ void
 all_windows_notify_cat_files_deleted (const char *catalog_path,
 				      GList      *list)
 {
-	gth_monitor_notify_update_cat_files (monitor,
+	gth_monitor_notify_update_cat_files (gth_monitor,
 					     catalog_path,
 					     GTH_MONITOR_EVENT_DELETED,
 					     list);
@@ -771,21 +786,21 @@ all_windows_notify_cat_files_deleted (const char *catalog_path,
 void
 all_windows_notify_files_created (GList *list)
 {
-	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CREATED, list);
+	gth_monitor_notify_update_files (gth_monitor, GTH_MONITOR_EVENT_CREATED, list);
 }
 
 
 void
 all_windows_notify_files_deleted (GList *list)
 {
-	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_DELETED, list);
+	gth_monitor_notify_update_files (gth_monitor, GTH_MONITOR_EVENT_DELETED, list);
 }
 
 
 void
 all_windows_notify_files_changed (GList *list)
 {
-	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CHANGED, list);
+	gth_monitor_notify_update_files (gth_monitor, GTH_MONITOR_EVENT_CHANGED, list);
 }
 
 
@@ -793,7 +808,7 @@ void
 all_windows_notify_file_rename (const gchar *old_name,
 				const gchar *new_name)
 {
-	gth_monitor_notify_file_renamed (monitor, old_name, new_name);
+	gth_monitor_notify_file_renamed (gth_monitor, old_name, new_name);
 }
 
 
@@ -807,7 +822,7 @@ all_windows_notify_files_rename (GList *old_names,
 		const char *old_name = o_scan->data;
 		const char *new_name = n_scan->data;
 
-		gth_monitor_notify_file_renamed (monitor, old_name, new_name);
+		gth_monitor_notify_file_renamed (gth_monitor, old_name, new_name);
 		all_windows_notify_file_rename (old_name, new_name);
 
 		o_scan = o_scan->next;
@@ -820,21 +835,21 @@ void
 all_windows_notify_directory_rename (const gchar *old_name,
 				     const gchar *new_name)
 {
-	gth_monitor_notify_directory_renamed (monitor, old_name, new_name);
+	gth_monitor_notify_directory_renamed (gth_monitor, old_name, new_name);
 }
 
 
 void
 all_windows_notify_directory_delete (const char *path)
 {
-	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_DELETED);
+	gth_monitor_notify_update_directory (gth_monitor, path, GTH_MONITOR_EVENT_DELETED);
 }
 
 
 void
 all_windows_notify_directory_new (const char *path)
 {
-	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_CREATED);
+	gth_monitor_notify_update_directory (gth_monitor, path, GTH_MONITOR_EVENT_CREATED);
 }
 
 
@@ -842,49 +857,49 @@ void
 all_windows_notify_catalog_rename (const char *old_name,
 				   const char *new_name)
 {
-	gth_monitor_notify_catalog_renamed (monitor, old_name, new_name);
+	gth_monitor_notify_catalog_renamed (gth_monitor, old_name, new_name);
 }
 
 
 void
 all_windows_notify_catalog_new (const gchar *path)
 {
-	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CREATED);
+	gth_monitor_notify_update_catalog (gth_monitor, path, GTH_MONITOR_EVENT_CREATED);
 }
 
 
 void
 all_windows_notify_catalog_delete (const gchar *path)
 {
-	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_DELETED);
+	gth_monitor_notify_update_catalog (gth_monitor, path, GTH_MONITOR_EVENT_DELETED);
 }
 
 
 void
 all_windows_notify_catalog_reordered (const gchar *path)
 {
-	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CHANGED);
+	gth_monitor_notify_update_catalog (gth_monitor, path, GTH_MONITOR_EVENT_CHANGED);
 }
 
 
 void
 all_windows_notify_update_metadata (const gchar *filename)
 {
-	gth_monitor_notify_update_metadata (monitor, filename);
+	gth_monitor_notify_update_metadata (gth_monitor, filename);
 }
 
 
 void
 all_windows_notify_update_icon_theme (void)
 {
-	gth_monitor_notify_update_icon_theme (monitor);
+	gth_monitor_notify_update_icon_theme (gth_monitor);
 }
 
 
 gboolean
 all_windows_remove_monitor (void)
 {
-	gth_monitor_pause (monitor);
+	gth_monitor_pause (gth_monitor);
 	return FALSE;
 }
 
@@ -892,7 +907,7 @@ all_windows_remove_monitor (void)
 gboolean
 all_windows_add_monitor (void)
 {
-	gth_monitor_resume (monitor);
+	gth_monitor_resume (gth_monitor);
 	return FALSE;
 }
 
@@ -904,40 +919,23 @@ get_fs_icon (IconName icon_name,
 	GdkPixbuf *pixbuf = NULL;
 	gboolean   scale = TRUE;
 
-	/*
-	if (icon_pixbuf[icon_name] != NULL) {
-		g_object_ref (icon_pixbuf[icon_name]);
-		return icon_pixbuf[icon_name];
-	}
-
-	icon_pixbuf[icon_name] = create_pixbuf (icon_theme,
-						icon_mime_name[icon_name],
-						icon_size);
-
-	return icon_pixbuf[icon_name];
-	*/
-
 	if (icon_pixbuf[icon_name] == NULL) {
-		const GnomeIconData *icon_data;
-		int                  base_size;
-		char                *icon_path;
+		GtkIconInfo         *icon_info = NULL;
 
-		icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-							  icon_mime_name[icon_name],
-							  icon_size,
-							  &icon_data,
-							  &base_size);
+		icon_info = gtk_icon_theme_lookup_icon (icon_theme,
+							icon_mime_name[icon_name],
+							icon_size,
+							0);
 
-		if (icon_path == NULL) {
+		if (icon_info == NULL) {
 			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1,
 									     dir_16_rgba,
 									     FALSE,
 									     NULL);
 			scale = FALSE;
 		} else {
-			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_file (icon_path,
-									   NULL);
-			g_free (icon_path);
+			icon_pixbuf[icon_name] = gtk_icon_info_load_icon (icon_info, NULL);
+			gtk_icon_info_free (icon_info);
 		}
 	}
 
@@ -981,8 +979,8 @@ get_folder_pixbuf_size_for_list (GtkWidget *widget)
 	int icon_width, icon_height;
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           GTK_ICON_SIZE_SMALL_TOOLBAR,
-                                           &icon_width, &icon_height);
+					   GTK_ICON_SIZE_SMALL_TOOLBAR,
+					   &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
 
@@ -993,8 +991,8 @@ get_folder_pixbuf_size_for_menu (GtkWidget *widget)
 	int icon_width, icon_height;
 
 	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-                                           GTK_ICON_SIZE_MENU,
-                                           &icon_width, &icon_height);
+					   GTK_ICON_SIZE_MENU,
+					   &icon_width, &icon_height);
 	return MAX (icon_width, icon_height);
 }
 
@@ -1005,7 +1003,8 @@ folder_is_film (const char *folder)
 	CommentData *cdata;
 	gboolean     film = FALSE;
 
-	folder = remove_scheme_from_uri (folder);
+	if (! is_local_file (folder))
+		return FALSE;
 
 	cdata = comments_load_comment (folder, FALSE);
 	if (cdata != NULL) {
